# -*- coding: utf-8 -*-
#
# name:             test_zone.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       06/13/2019
#

"""
Functional tests for cloudns_api's zone module.
"""

from cloudns_api import zone

from .helpers import mock_get_request, mock_post_request


##
# Zone Tests

@mock_get_request()
def test_zone_list_function():
    """Zone list function sends properly formated request."""
    response = zone.list()
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/list-zones.json'
    assert payload['params']['page'] == 1
    assert payload['params']['rows-per-page'] == 10
    assert payload['params']['search'] == ''
    assert payload['params']['group-id'] == ''


@mock_get_request()
def test_zone_get_page_count_function():
    """Zone get_page_count function sends properly formated request."""
    response = zone.get_page_count(rows_per_page=10)
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/get-pages-count.json'
    assert payload['params']['rows-per-page'] == 10
    assert payload['params']['search'] == ''
    assert payload['params']['group-id'] == ''


@mock_post_request()
def test_zone_create_function():
    """Zone create function sends properly formated request."""
    response = zone.create(domain_name='example.com', zone_type='Master',
                           ns=[])
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/register.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['zone-type'] == 'Master'
    assert payload['params']['ns'] == []


@mock_post_request()
def test_soa_create_function_catches_validation_errors():
    """SOA update function catches validation errors."""
    response = zone.create('not a domain', zone_type='Master')

    assert not response.success
    assert response.json()['error'] == 'Validation error.'


@mock_get_request()
def test_zone_get_function():
    """Zone get function sends properly formated request."""
    response = zone.get(domain_name='example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/get-zone-info.json'
    assert payload['params']['domain-name'] == 'example.com'


@mock_post_request()
def test_zone_update_function():
    """Zone update function sends properly formated request."""
    response = zone.update(domain_name='example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/update-zone.json'
    assert payload['params']['domain-name'] == 'example.com'


@mock_post_request()
def test_zone_activate_function():
    """Zone actiate function sends properly formated request."""
    response = zone.activate(domain_name='example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/change-status.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['status'] == 1


@mock_post_request()
def test_zone_deactivate_function():
    """Zone deactivate function sends properly formated request."""
    response = zone.deactivate(domain_name='example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/change-status.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['status'] == 0


@mock_post_request()
def test_zone_toggle_activation_function():
    """Zone toggle_activation function sends properly formated request."""
    response = zone.toggle_activation(domain_name='example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/change-status.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert 'status' not in payload['params']


@mock_post_request()
def test_zone_delete_function():
    """Zone delete function sends properly formated request."""
    response = zone.delete(domain_name='example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/delete.json'
    assert payload['params']['domain-name'] == 'example.com'


@mock_get_request()
def test_zone_get_stats_function():
    """Zone get_stats function sends properly formated request."""
    response = zone.get_stats()
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/get-zones-stats.json'


@mock_get_request()
def test_dnssec_available():
    """Check if DNSSEC is available for the provided zone"""
    response = zone.dnssec_available('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/is-dnssec-available.json'  # noqa: E501


@mock_post_request()
def test_dnssec_activate():
    """Activate DNSSEC for a zone."""
    response = zone.dnssec_activate('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/activate-dnssec.json'


@mock_post_request()
def test_dnssec_deactivate():
    """Deactivate DNSSEC for a zone."""
    response = zone.dnssec_deactivate('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/deactivate-dnssec.json'  #noqa: E501


@mock_get_request()
def test_dnssec_ds_records():
    """Get DNSSEC DS records for a zone."""
    response = zone.dnssec_ds_records('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/get-dnssec-ds-records.json'  #noqa: E501

@mock_get_request()
def test_zone_is_updated():
    """Check if all DNS servers have been updated."""
    response = zone.is_updated('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/is-updated.json'  #noqa: E501



# -*- coding: utf-8 -*-
#
# name:             tests/__init__.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       06/13/2019
#

"""
Test module for cloudns_api unit tests.
"""

from os import environ


environ['CLOUDNS_API_DEBUG'] = 'False'
environ['CLOUDNS_API_TESTING'] = 'True'
environ['CLOUDNS_API_AUTH_ID'] = 'test_auth_id'
environ['CLOUDNS_API_AUTH_PASSWORD'] = 'test_auth_password'



# -*- coding: utf-8 -*-
#
# name:             cloudns_api/__init__.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/15/2018
#

"""
cloudns_api
~~~~~~~~~~~

This package contains interface functions to make calls to the ClouDNS.net API.
"""

import requests  # noqa: F401

from . import api     # noqa: F401
from . import record  # noqa: F401
from . import soa     # noqa: F401
from . import zone    # noqa: F401


__title__ = 'cloudns_api'
__version__ = '0.9.2'
__author__ = 'Harold Bradley III | Prestix Studio, LLC'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018-2020 Harold Bradley III | Prestix Studio, LLC'

# Soli Deo gloria. <><



# -*- coding: utf-8 -*-
#
# name:             validation.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/16/2018
#

"""
cloudns_api.validation
~~~~~~~~~~~~~~~~~~~~~~

This module contains functions to validate fields passed into the API.
"""

import re


class ValidationError(Exception):
    """Exception thrown when a validation error has occured."""
    def __init__(self, fieldname, message, *args, **kwargs):
        """Initialize ValidationError with `fieldname` and `message` values."""
        self.details = {
            'fieldname': fieldname,
            'message':   message,
        }
        super(ValidationError, self).__init__(message, *args, **kwargs)

    def get_details(self):
        """Returns a list of error details."""
        return [self.details]


class ValidationErrorsBatch(ValidationError):
    """Exception thrown when multiple validation errors have occured."""
    def __init__(self, validation_errors, *args, **kwargs):
        """Initialize ValidationError with `validation_errors` list."""
        self.validation_errors = validation_errors
        super(ValidationError, self).__init__('Validation errors occured.',
                                              *args, **kwargs)

    def get_details(self):
        """Returns a list of error details."""
        return [error.details for error in self.validation_errors]


def validate(fieldname, value, optional=False, validate_as=None, *args,
             **kwargs):
    """Validates a value for a particular fieldtype.
    Returns True if it is valid; raises an error otherwise.

    :param fieldname: string, the fieldname to validate (determines the type of
        validation to use)
    :param value: mixed, the value to validate
    :param optional: bool, if True, accept None as a valid value
    :param validate_as: string, an optional argument to specifically select a
        validation function to run. If this is not passed, the fieldname will
        be used.
    """
    if optional and not value:
        return True
    elif value is None or value == '':
        raise ValidationError(fieldname, 'This field (' + fieldname +
                              ') is required.')

    if not validate_as:
        validate_as = fieldname

    if validate_as not in validation_functions:
        return True

    if validation_functions[validate_as](value, fieldname, *args, **kwargs):
        return True

    # If for some reason the validation fails, but no error was raised,
    # raise one here. This should hopefully never happen
    raise ValidationError('Unexpected validation error.')  # pragma: no cover


##
# Specific Validation Functions

def is_algorithm(value, fieldname='algorithm', **kwargs):
    """Validates and returns True if the value is a proper algorithm.
    Otherwise, raises a validation error."""
    try:
        if value.upper() not in ALGORITHMS:
            raise ValidationError(fieldname,
                                  'This field must be RSA, DSA, ECDSA, or ' +
                                  'Ed25519.')
    # If value isn't a string, upper() raises AttributeError
    except AttributeError:
        if value not in [1, 2, 3, 4]:
            raise ValidationError(fieldname,
                                  'This field must be RSA, DSA, ECDSA, or ' +
                                  'Ed25519.')
    return True


ALGORITHMS = ['RSA', 'DSA', 'ECDSA', 'ED25519']


def is_api_bool(value, fieldname='api-bool', **kwargs):
    """Validates and returns True if the value is a 0 or 1. Otherwise, raises a
    validation error."""
    if value != 0 and value != 1:
        raise ValidationError(fieldname,
                              'This field must be 0 or 1.')
    return True


def is_caa_flag(value, fieldname='caa-flag', **kwargs):
    """Validates and returns True if the value is 0 or 128. Otherwise, raises a
    validation error."""
    if value != 0 and value != 128:
        raise ValidationError(fieldname,
                              'This field must be 0 (non-critical) or 128 ' +
                              '(critical).')
    return True


def is_caa_type(value, fieldname='caa-type', **kwargs):
    """Validates and returns True if value is a caa type. Otherwise, raises a
    validation error."""
    try:
        if value.lower() not in CAA_TYPES:
            raise ValidationError(fieldname,
                                  'This field must be one of issue, ' +
                                  'issuewild, iodef.')
    # If value isn't a string, lower() raises AttributeError
    except AttributeError:
        raise ValidationError(fieldname,
                              'This field must be one of issue, ' +
                              'issuewild, iodef.')
    return True


CAA_TYPES = ['issue', 'issuewild', 'iodef']


def is_domain_name(value, fieldname='domain-name', **kwargs):
    """Validates and returns True if the value is a valid domain name.
    Otherwise, raises a validation error."""
    if not re.match(
       r'^((?=[a-z0-9-]{1,63}\.)(xn--)?[a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,63}$',
       value):
        raise ValidationError(fieldname,
                              'This field must be a valid domain name.')
    return True


def is_email(value, fieldname='email', **kwargs):
    """Validates and returns true if the value is a valid email. Otherwise,
    raises a validation error."""
    if not re.match(r'(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)',
                    value):
        raise ValidationError(fieldname,
                              'This field must be a valid email.')
    return True


def is_fptype(value, fieldname='fptype', **kwargs):
    """Validates and returns True if the value is a proper fingerprint type.
    Otherwise, raises a validation error."""
    try:
        if value.upper() not in FP_TYPES:
            raise ValidationError(fieldname,
                                  'This field must be one of SHA-1 or ' +
                                  'SHA-256.')
    # If value isn't a string, upper() raises AttributeError
    except AttributeError:
        if value not in [1, 2]:
            raise ValidationError(fieldname,
                                  'This field must be one of SHA-1 or ' +
                                  'SHA-256.')
    return True


FP_TYPES = ['SHA-1', 'SHA-256']


def is_int(value, fieldname='int', min_value=None, max_value=None, **kwargs):
    """Validates and returns True if the value is an integer (within
    min_value/max_value range). Otherwise, raises a validation error."""
    try:
        value += 0  # Try it to see if it is an integer
    except TypeError:
        if re.findall('[^0-9]', value):
            raise ValidationError(fieldname, 'This field must be an integer.')

    if min_value and int(value) < min_value:
        raise ValidationError(fieldname,
                              'This field must be greater than ' +
                              str(min_value) + '.')
    if max_value and int(value) > max_value:
        raise ValidationError(fieldname,
                              'This field must be less than ' +
                              str(max_value) + '.')
    return True


def is_ipv4(value, fieldname='ipv4', **kwargs):
    """Validates and returns True if the value is an IPv4 address. Otherwise,
    raises a validation error. Note that this is just a rudimentary check."""
    octets = []

    if hasattr(value, 'split'):
        octets = [o for o in value.split('.') if o.isdigit() and
                  0 <= int(o) and int(o) <= 255]

    if len(octets) != 4:
        raise ValidationError(fieldname,
                              'This field must be a valid IPv4 address.')

    return True


def is_ipv6(value, fieldname='ipv6', **kwargs):
    """Validates and returns True if the value is an IPv6 address. Otherwise,
    raises a validation error. Note that this is just a rudimentary check."""
    hextet = []

    if hasattr(value, 'split'):
        try:
            hextet = [h for h in value.split(':')
                      if 0 <= int(h, 16) and int(h, 16) <= 65535]
        except ValueError:
            hextet = []

    if len(hextet) != 8:
        raise ValidationError(fieldname,
                              'This field must be a valid IPv6 address.')

    return True


def is_record_type(value, fieldname='record-type', **kwargs):
    """Validates and returns True if the value is a valid domain record type.
    Otherwise, raises a validation error."""
    try:
        if value.upper() not in RECORD_TYPES:
            raise ValidationError(fieldname,
                                  'This field must be a valid domain record' +
                                  ' type.')
    # If value isn't a string, upper() raises AttributeError
    except AttributeError:
        raise ValidationError(fieldname,
                              'This field must be a valid domain record type.')
    return True


RECORD_TYPES = ['A', 'AAAA', 'MX', 'CNAME', 'TXT', 'SPF', 'NS', 'SRV', 'WR',
                'RP', 'SSHFP', 'ALIAS', 'CAA', 'NAPTR', 'PTR']


def is_redirect_type(value, fieldname='redirect-type', **kwargs):
    """Validates and returns True if the value is 301 or 302. Otherwise, raises
    a validation error."""
    if value != 301 and value != 302:
        raise ValidationError(fieldname,
                              'This field must be 301 (permanent) or 302 ' +
                              '(temporary).')
    return True


def is_required(value, fieldname='required', **kwargs):
    """Validates and returns True if any value is provided. Otherwise, raises a
    validation error."""
    if value is None or value == '':
        raise ValidationError(fieldname,
                              'This field is required.')
    return True


def is_rows_per_page(value, fieldname='rows-per-page', **kwargs):
    """Validates and returns True if a proper rows-per-page value is provided
    (10, 20, 30, 50, or 100). Otherwise, raises a valiadation error."""
    if value not in [10, 20, 30, 50, 100, '10', '20', '30', '50', '100']:
        raise ValidationError(fieldname,
                              'This field must be one of: 10, 20, 30, 50, or' +
                              ' 100.')
    return True


def is_tlsa_matching_type(value, fieldname='tlsa_matching_type', **kwargs):
    """Validates and returns True if a proper tlsa_matching_type value is
    provided (0, 1, 2). Otherwise, raises a valiadation error."""
    if value not in [0, 1, 2, '0', '1', '2']:
        raise ValidationError(fieldname,
                              'This field must be one of: 0, 1, or 2')
    return True


def is_tlsa_selector(value, fieldname='tlsa_selector', **kwargs):
    """Validates and returns True if a proper tlsa_selector value is provided
    (0, 1). Otherwise, raises a valiadation error."""
    if value not in [0, 1, '0', '1']:
        raise ValidationError(fieldname,
                              'This field must be one of: 0 or 1')
    return True


def is_tlsa_usage(value, fieldname='tlsa_usage', **kwargs):
    """Validates and returns True if a proper tlsa_usage value is provided (0,
    1, 2, 3). Otherwise, raises a valiadation error."""
    if value not in [0, 1, 2, 3, '0', '1', '2', '3']:
        raise ValidationError(fieldname,
                              'This field must be one of: 0, 1, 2, or 3')
    return True


def is_ttl(value, fieldname='ttl', **kwargs):
    """Validates and returns True if the value is a valid ClouDNS ttl.
    Otherwise, raises a validation error."""
    if hasattr(value, 'lower'):
        value = value.lower()

    if value not in TTLS and value not in TTL_STRINGS:
        raise ValidationError(fieldname, 'This field must be a valid ttl. ' +
                              '(1 minute, 5 minutes, 15 minutes, 30 minutes,' +
                              ' 1 hour, 6 hours, 12 hours, 1 day, 2 days, 3 ' +
                              'days, 1 week, 2 weeks, or 1 month) or (60, ' +
                              '300, 900, 1800, 3600, 21600, 43200, 86400, ' +
                              '172800, 259200, 604800, 1209600, or 2592000)')
    return True


TTL_STRINGS = ['1 minute', '5 minutes', '15 minutes', '30 minutes', '1 hour',
               '6 hours', '12 hours', '1 day', '2 days', '3 days', '1 week',
               '2 weeks', '1 month', '60', '300', '900', '1800', '3600',
               '21600', '43200', '86400', '172800', '259200', '604800',
               '1209600', '2592000']

TTLS = [60, 300, 900, 1800, 3600, 21600, 43200, 86400, 172800, 259200, 604800,
        1209600, 2592000]


def is_valid(value, fieldname='valid', **kwargs):
    """This is a stub. It always returns true."""
    return True


def is_zone_type(value, fieldname='zone-type', **kwargs):
    """Validates and returns True if the value is a valid zone type.
    Otherwise, raises a validation error."""
    if hasattr(value, 'lower'):
        value = value.lower()

    if value not in ZONE_TYPES:
        raise ValidationError(fieldname, 'This field must be a valid zone ' +
                              'type. (master, slave, parked, or geodns)')
    return True


ZONE_TYPES = ['master', 'slave', 'parked', 'geodns', 'domain', 'reverse']


# Set up validation functions dict
validation_functions = {
    'admin-mail':          is_email,
    'algorithm':           is_algorithm,
    'bool':                is_api_bool,
    'caa_flag':            is_caa_flag,
    'caa_type':            is_caa_type,
    'caa_value':           is_valid,
    'default-ttl':         is_ttl,
    'domain-name':         is_domain_name,
    'email':               is_email,
    'fptype':              is_fptype,
    'frame':               is_api_bool,
    'frame-title':         is_valid,
    'geodns-location':     is_int,
    'integer':             is_int,
    'ipv4':                is_ipv4,
    'ipv6':                is_ipv6,
    'mail':                is_email,
    'page':                is_int,
    'port':                is_int,
    'primary-ns':          is_domain_name,
    'priority':            is_int,
    'record':              is_required,
    'record-id':           is_int,
    'record-type':         is_record_type,
    'redirect-type':       is_redirect_type,
    'refresh':             is_int,
    'required':            is_required,
    'rows-per-page':       is_rows_per_page,
    'save-path':           is_api_bool,
    'status':              is_api_bool,
    'tlsa_matching_type':  is_tlsa_matching_type,
    'tlsa_selector':       is_tlsa_selector,
    'tlsa_usage':          is_tlsa_usage,
    'ttl':                 is_ttl,
    'txt':                 is_domain_name,
    'valid':               is_valid,
    'weight':              is_int,
    'zone-type':           is_zone_type,
}



# -*- coding: utf-8 -*-
#
# name:             soa.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/16/2018
#

"""
cloudns_api.soa
~~~~~~~~~~~~~~~

This module contains API wrapper functions for listing and updating SOA
records.
"""

import requests

from .api import api, patch_update
from .parameters import Parameters


@api
def get(domain_name=None):
    """Retreives the DNS SOA record for a particular domain.

    :param domain_name: string, (required) the domain name for which to
        retrieve the SOA record
    """
    url = 'https://api.cloudns.net/dns/soa-details.json'

    params = Parameters({'domain-name': domain_name})

    return requests.get(url, params=params.to_dict())


@api
@patch_update(get=get, keys=['domain_name'])
def update(domain_name=None, primary_ns=None, admin_mail=None, refresh=None,
           retry=None, expire=None, default_ttl=None, patch=False, **kwargs):
    """Updates the DNS SOA record for a particular domain.

    :param domain_name: string, (required) the domain name whose SOA record you
        want to update
    :param primary_ns: string, (required) hostname of primary nameserver
    :param admin_mail: string, (required) DNS admin's e-mail
    :param refresh: integer, (required) refresh rate from 1200 to 43200 seconds
    :param retry: integer, (required) retry rate from 180 to 2419200 seconds
    :param expire: integer, (required) expire time from 1209600 to 2419200
        seconds
    :param default_ttl: integer, (required) default TTL from 60 to 2419200
        seconds
    :param patch: boolean, whether or not to do a patch update
    """
    url = 'https://api.cloudns.net/dns/modify-soa.json'

    params = Parameters({
            'domain-name': domain_name,
            'primary-ns': primary_ns,
            'admin-mail': admin_mail,
            'refresh': {
                'value': refresh,
                'min_value': 1200,
                'max_value': 43200,
            },
            'retry': {
                'value': retry,
                'min_value': 180,
                'max_value': 2419200,
            },
            'expire': {
                'value': expire,
                'min_value': 1209600,
                'max_value': 2419200,
            },
            'default-ttl': {
                'value': default_ttl,
                'min_value': 60,
                'max_value': 2419200,
            },
        })

    return requests.post(url, params=params.to_dict())


def patch(*args, **kwargs):
    """A convenience function for patch updates."""
    return update(*args, patch=True, **kwargs)



# -*- coding: utf-8 -*-
#
# name:             parameters.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       05/26/2019
#

"""
cloud_api.parameters
~~~~~~~~~~~~~~~~~~~~

This module contains the Parameters class for self-validating parameters
objects.

The object allows for declaring parameters with validation options for use in
self-validating (or manual validation).
"""

from .api import get_auth_params
from .validation import validate, ValidationError, ValidationErrorsBatch


class Parameters(object):
    def __init__(self, params_with_options, validate=True):
        """Initializes the Parameters object

        :param params_with_options: dict, a dict of parameters with their
            options for validating.
            Format without options:
                {
                    'fieldname' : value,
                    ...
                }
            Format with options:
                {
                    # Will be validated as 'fieldname'
                    'fieldname' : {
                        'value': value,
                        'optional': is_optional,
                        ...
                    },
                    # Will be validated as 'valid'
                    'fieldname_2' : {
                        'value': value,
                        'validate_as': 'valid',
                        ...
                    },
                    ...
                }
        :param validate: boolean, whether or not to validate parameters on
            initialization. Defaults to True.
        """
        self._params_with_options = {**get_auth_params(),
                                     **params_with_options}

        if (validate):
            self.validate()

    def to_dict(self):
        """Returns a dict of just the fieldname and values."""
        return {fieldname: options['value']
                if isinstance(options, dict) else options
                for fieldname, options in self._params_with_options.items()}

    def validate(self):
        """Validates the parameters according to the fieldname and any optional
        validation options."""
        errors = []

        for fieldname, options in self._params_with_options.items():
            if isinstance(options, dict):
                options = options.copy()
                value = options.pop('value')
            else:
                # Options variable is actually the value in this instance:
                value = options
                options = {}

            try:
                validate(fieldname, value, **options)
            except ValidationError as e:
                errors.append(e)

        if len(errors) == 1:
            raise errors[0]
        elif len(errors) > 1:
            raise ValidationErrorsBatch(errors)



# -*- coding: utf-8 -*-
#
# name:             test_api.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       05/26/2019
#

"""
Unit tests for cloudns_api's api soa module.
"""

from cloudns_api import soa

from .helpers import mock_get_request, mock_post_request


##
# SOA Tests

@mock_get_request()
def test_soa_get_function():
    """SOA get function sends properly formated request."""
    response = soa.get('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/soa-details.json'
    assert payload['params']['domain-name'] == 'example.com'


@mock_post_request()
def test_soa_update_function():
    """SOA update function sends properly formated update request."""
    response = soa.update('example.com',
                          primary_ns='ns1.example.com',
                          admin_mail='test@example.com', refresh=1200,
                          retry=180, expire=1209600, default_ttl=60)

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/modify-soa.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['primary-ns'] == 'ns1.example.com'
    assert payload['params']['admin-mail'] == 'test@example.com'
    assert payload['params']['refresh'] == 1200
    assert payload['params']['retry'] == 180
    assert payload['params']['expire'] == 1209600
    assert payload['params']['default-ttl'] == 60


@mock_get_request(payload={
    'serialNumber':  '2019051202',
    'primaryNS':     'ns1.prestix.host',
    'adminMail':     'support@prestix.host',
    'refresh':       '7200',
    'retry':         '1800',
    'expire':        '1209600',
    'defaultTTL':    '3600',
})
@mock_post_request()
def test_soa_update_function_using_patch():
    """SOA update function sends properly formated update request when doing a
    patch update."""
    response = soa.update(domain_name='example.com',
                          admin_mail='new_email@example.com', patch=True)

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/modify-soa.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['admin-mail'] == 'new_email@example.com'


@mock_get_request(payload={
    'serialNumber':  '2019051202',
    'primaryNS':     'ns1.prestix.host',
    'adminMail':     'support@prestix.host',
    'refresh':       '7200',
    'retry':         '1800',
    'expire':        '1209600',
    'defaultTTL':    '3600',
})
@mock_post_request()
def test_soa_update_function_using_patch_helper():
    """SOA update function sends properly formated update request when doing a
    patch update using the patch helper function."""
    response = soa.patch(domain_name='example.com',
                         admin_mail='new_email@example.com')

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/modify-soa.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['admin-mail'] == 'new_email@example.com'


@mock_post_request()
def test_soa_update_function_catches_validation_errors():
    """SOA update function catches validation errors."""
    response = soa.update('example.com',
                          primary_ns='not a domain',
                          admin_mail='test@example.com', refresh=1200,
                          retry=180, expire=1209600, default_ttl=60)

    assert not response.success
    assert response.json()['error'] == 'Validation error.'



# -*- coding: utf-8 -*-
#
# name:             test_record.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       05/25/2019
#

"""
Functional tests for cloudns_api's record module.
"""

from pytest import raises

from cloudns_api import record
from cloudns_api.validation import ValidationError

from .helpers import mock_get_request, mock_post_request


##
# Record Tests

@mock_get_request()
def test_record_get_available_record_types_function():
    """Record get_available_record_types function sends properly formated
    request."""
    response = record.get_available_record_types('domain')
    assert response.success

    payload = response.payload
    assert payload['url'] == \
        'https://api.cloudns.net/dns/get-available-record-types.json'
    assert payload['params']['zone-type'] == 'domain'


@mock_get_request()
def test_record_get_available_record_types_catches_validation_errors():
    """Record get_available_record_types function catches invalid record
    type."""
    response = record.get_available_record_types('not-valid')

    assert not response.success
    assert response.json()['error'] == 'Validation error.'


@mock_get_request()
def test_record_get_available_ttls_function():
    """Record get_available_ttls function sends properly formated request."""
    response = record.list('example.com', host='ns1')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/records.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['host'] == 'ns1'


@mock_get_request()
def test_record_list_function():
    """Record list function sends properly formated request."""
    response = record.list('example.com', host='ns1')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/records.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['host'] == 'ns1'


def test_generate_record_parameters_works_for_a_records():
    """The generate_record_parameters function generates A record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='', record_type='A',
        record='10.10.10.10', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'A'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == ''
    assert payload['record'] == '10.10.10.10'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_a_record_errors():
    """The generate_record_parameters function catches A record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='', record_type='A', ttl=3600,
            record='not an ip address')


def test_generate_record_parameters_works_for_aaaa_records():
    """The generate_record_parameters function generates AAAA record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='', record_type='AAAA',
        record='2009:0db8:95a3:0000:0000:9a2e:0370:8334', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'AAAA'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == ''
    assert payload['record'] == '2009:0db8:95a3:0000:0000:9a2e:0370:8334'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_aaaa_record_errors():
    """The generate_record_parameters function catches AAAA record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='', record_type='AAAA', ttl=3600,
            record='not an ip address')


def test_generate_record_parameters_works_for_mx_records():
    """The generate_record_parameters function generates MX record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='mx1', record_type='MX',
        record='mail.example.com', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'MX'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == 'mx1'
    assert payload['record'] == 'mail.example.com'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_mx_record_errors():
    """The generate_record_parameters function catches MX record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='mx1', record_type='MX', ttl=3600,
            record='not an hostname')


def test_generate_record_parameters_works_for_cname_records():
    """The generate_record_parameters function generates CNAME record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='www', record_type='CNAME',
        record='example.com', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'CNAME'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == 'www'
    assert payload['record'] == 'example.com'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_cname_record_errors():
    """The generate_record_parameters function catches CNAME record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='www', record_type='CNAME',
            ttl=3600, record='not an hostname')


def test_generate_record_parameters_works_for_txt_records():
    """The generate_record_parameters function generates TXT record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='', record_type='TXT',
        record='Anything goes...', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'TXT'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == ''
    assert payload['record'] == 'Anything goes...'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_works_for_spf_records():
    """The generate_record_parameters function generates SPF record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='', record_type='SPF',
        record='Anything goes...', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'SPF'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == ''
    assert payload['record'] == 'Anything goes...'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_works_for_ns_records():
    """The generate_record_parameters function generates NS record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='www', record_type='NS',
        record='example.com', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'NS'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == 'www'
    assert payload['record'] == 'example.com'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_ns_record_errors():
    """The generate_record_parameters function catches NS record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='www', record_type='NS',
            ttl=3600, record='not an hostname')


def test_generate_record_parameters_works_for_srv_records():
    """The generate_record_parameters function generates SRV record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='srv1', record_type='SRV',
        record='srv.example.org', priority=10, weight=10, ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'SRV'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == 'srv1'
    assert payload['record'] == 'srv.example.org'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_srv_record_errors():
    """The generate_record_parameters function catches SRV record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='srv', record_type='SRV',
            priority=10, weight=10, ttl=3600, record='not an hostname')


def test_generate_record_parameters_works_for_wr_records():
    """The generate_record_parameters function generates WR record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='', record_type='WR',
        redirect_type=301, record='https://example.org/path/', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'WR'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == ''
    assert payload['record'] == 'https://example.org/path/'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_works_for_alias_records():
    """The generate_record_parameters function generates ALIAS record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='www', record_type='ALIAS',
        record='www.example.org', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'ALIAS'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == 'www'
    assert payload['record'] == 'www.example.org'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_alias_record_errors():
    """The generate_record_parameters function catches ALIAS record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='www', record_type='ALIAS',
            ttl=3600, record='not an hostname')


def test_generate_record_parameters_works_for_rp_records():
    """The generate_record_parameters function generates RP record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='', record_type='RP',
        record='an.email@example.com', ttl=3600)

    payload = parameters.to_dict()

    assert payload['record-type'] == 'RP'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == ''
    assert payload['record'] == 'an.email@example.com'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_works_for_sshfp_records():
    """The generate_record_parameters function generates SSHFP record
    parameters."""
    parameters = record.generate_record_parameters(
        domain_name='example.com', host='www', record_type='SSHFP',
        record='the fingerprint...', ttl=3600, algorithm='RSA',
        fptype='SHA-256')

    payload = parameters.to_dict()

    assert payload['record-type'] == 'SSHFP'
    assert payload['domain-name'] == 'example.com'
    assert payload['host'] == 'www'
    assert payload['record'] == 'the fingerprint...'
    assert payload['ttl'] == 3600


def test_generate_record_parameters_catches_sshfp_record_errors():
    """The generate_record_parameters function catches SSHFP record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', host='www', record_type='SSHFP',
            record='the fingerprint...', ttl=3600,
            algorithm='not an algorithm', fptype='SHA-356')


def test_generate_record_parameters_checks_for_invalid_type():
    """The generate_record_parameters function catches NS record errors."""
    with raises(ValidationError):
        record.generate_record_parameters(
            domain_name='example.com', record_type='bad-type')


@mock_post_request()
def test_record_create_function():
    """Record create function sends properly formated update request."""
    response = record.create('example.com', host='', record_type='A',
                             record='10.10.10.10', ttl=3600)

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/add-record.json'
    assert payload['params']['record-type'] == 'A'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['host'] == ''
    assert payload['params']['record'] == '10.10.10.10'
    assert payload['params']['ttl'] == 3600


@mock_post_request()
def test_record_create_function_catches_validation_errors():
    """Record create function catches validation errors."""
    response = record.create('not a valid domain', host='', record_type='A',
                             record='10.10.10.10', ttl=3600)

    assert not response.success
    assert response.json()['error'] == 'Validation error.'


@mock_post_request()
def test_record_transfer_function():
    """Record transfer function sends properly formated get request."""
    response = record.transfer('example.com', server='1.1.1.1')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/axfr-import.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['server'] == '1.1.1.1'


@mock_get_request()
def test_record_copy_function():
    """Record copy function sends properly formated get request."""
    response = record.copy('example.com', from_domain='example.net',
                           delete_current_records=True)
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/copy-records.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['from-domain'] == 'example.net'
    assert payload['params']['delete-current-records'] == 1


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
def test_record_get_function():
    """Record get function sends properly formated get request."""
    response = record.get('example.com', 1234)

    assert response.success

    payload = response.payload

    assert payload['type'] == 'A'
    assert payload['domain-name'] == 'example.com'
    assert payload['record-id'] == 1234
    assert payload['host'] == 'ns1'
    assert payload['ttl'] == 3600
    assert payload['record'] == '10.0.0.10'


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
def test_record_get_function_with_bad_record_id():
    """Record get function sends properly formated error message when record_id
    does not exist."""
    response = record.get('example.com', 5678)

    assert not response.success
    assert response.error == \
        'Record "5678" not found in "example.com" zone.'
    assert str(response.status_code) == '404'


@mock_get_request()
def test_record_export_function():
    """Record export function sends properly formated get request."""
    response = record.export('example.com')
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/records-export.json'
    assert payload['params']['domain-name'] == 'example.com'


@mock_get_request()
def test_record_get_dynamic_url_function():
    """Record get_dynamic_url function sends properly formated get request."""
    response = record.get_dynamic_url('example.com', record_id=1234)
    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/get-dynamic-url.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == 1234


@mock_post_request()
def test_record_update_function():
    """Record update function sends properly formated update request."""
    response = record.update('example.com', record_id=1234, record_type='A',
                             host='ns1', ttl=3600, record='10.0.0.10')

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/mod-record.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == 1234
    assert payload['params']['host'] == 'ns1'
    assert payload['params']['ttl'] == 3600
    assert payload['params']['record'] == '10.0.0.10'


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
@mock_post_request()
def test_record_update_function_can_figure_out_record_type():
    """Record update function can figure out what record type a record
    is."""
    response = record.update('example.com', record_id=1234, host='ns1',
                             ttl=3600, record='10.0.0.10')

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/mod-record.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == 1234
    assert payload['params']['host'] == 'ns1'
    assert payload['params']['ttl'] == 3600
    assert payload['params']['record'] == '10.0.0.10'


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
def test_record_update_function_with_bad_record_id():
    """Record update function sends properly formated error message when record_id
    does not exist."""
    response = record.update('example.com', record_id=5678, host='ns1',
                             ttl=3600, record='10.0.0.10')

    assert not response.success
    assert response.error == \
        'Record "5678" not found in "example.com" zone.'
    assert str(response.status_code) == '404'


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
@mock_post_request()
def test_record_update_function_using_patch():
    """Record update function sends properly formated update request when doing a
    patch update."""
    response = record.update('example.com', record_id=1234,
                             record='10.10.10.10', patch=True)

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/mod-record.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == 1234
    assert payload['params']['host'] == 'ns1'
    assert payload['params']['ttl'] == 3600
    assert payload['params']['record'] == '10.10.10.10'


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
@mock_post_request()
def test_record_update_function_using_patch_helper():
    """Record update function sends properly formated update request when doing a
    patch update using the patch helper function."""
    response = record.patch('example.com', record_id=1234,
                            record='10.10.10.10')

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/mod-record.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == 1234
    assert payload['params']['host'] == 'ns1'
    assert payload['params']['ttl'] == 3600
    assert payload['params']['record'] == '10.10.10.10'


@mock_get_request(payload={
    1234: {
        'type':        'A',
        'domain-name': 'example.com',
        'record-id':   1234,
        'host':        'ns1',
        'ttl':         3600,
        'record':      '10.0.0.10',
    }
})
def test_record_patch_function_with_bad_record_id():
    """Record update function sends properly formated error message when record_id
    does not exist."""
    response = record.patch('example.com', record_id=5678, host='ns1',
                            ttl=3600, record='10.0.0.10')

    assert not response.success
    assert response.error == \
        'Record "5678" not found in "example.com" zone.'
    assert str(response.status_code) == '404'


@mock_post_request()
def test_record_update_function_catches_validation_errors():
    """Record update function catches validation errors."""
    response = record.update('example.com', record_id=1234, record_type='A',
                             host='ns1', ttl='not a ttl', record='10.0.0.10')

    assert not response.success
    assert response.json()['error'] == 'Validation error.'


@mock_post_request()
def test_record_activate_function():
    """Record actiate function sends properly formated request."""
    response = record.activate(domain_name='example.com', record_id='123')
    assert response.success

    payload = response.payload
    assert payload['url'] == \
        'https://api.cloudns.net/dns/change-record-status.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == '123'
    assert payload['params']['status'] == 1


@mock_post_request()
def test_record_deactivate_function():
    """Record deactivate function sends properly formated request."""
    response = record.deactivate(domain_name='example.com', record_id='123')
    assert response.success

    payload = response.payload
    assert payload['url'] == \
        'https://api.cloudns.net/dns/change-record-status.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == '123'
    assert payload['params']['status'] == 0


@mock_post_request()
def test_record_toggle_activation_function():
    """Record toggle_activation function sends properly formated request."""
    response = record.toggle_activation(domain_name='example.com',
                                        record_id='123')
    assert response.success

    payload = response.payload
    assert payload['url'] == \
        'https://api.cloudns.net/dns/change-record-status.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == '123'
    assert 'status' not in payload['params']


@mock_post_request()
def test_record_delete_function():
    """Record delete function sends properly formated update request."""
    response = record.delete('example.com', record_id='123456789')

    assert response.success

    payload = response.payload
    assert payload['url'] == 'https://api.cloudns.net/dns/delete-record.json'
    assert payload['params']['domain-name'] == 'example.com'
    assert payload['params']['record-id'] == '123456789'



# -*- coding: utf-8 -*-
#
# name:             test_api.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/16/2018
#

"""
Functional tests for cloudns_api's api utilities module.
"""

from mock import patch
from requests import exceptions as request_exceptions

from cloudns_api.api import (
    ApiResponse,
    ApiException,
    RequestResponseStub,
    api,
    get_auth_params,
    get_login,
    get_my_ip,
    get_nameservers,
    patch_update,
    use_snake_case_keys,
)
from cloudns_api.validation import ValidationError

from .helpers import (
    mock_get_request,
    set_debug,
    set_no_debug,
    use_test_auth,
)


##
# Authentication Tests

@use_test_auth
def test_get_auth_params_returns_auth_params(test_id, test_password):
    """Function get_auth_params() returns auth params."""
    auth_params = get_auth_params()
    assert auth_params['auth-id'] == test_id
    assert auth_params['auth-password'] == test_password


@patch('cloudns_api.api.CLOUDNS_API_SUB_AUTH_ID', new=123)
@patch('cloudns_api.api.CLOUDNS_API_AUTH_ID', new=None)
def test_get_auth_params_returns_sub_auth_id():
    """Function get_auth_params() returns sub auth params."""
    auth_params = get_auth_params()
    assert auth_params['sub-auth-id'] == 123
    assert 'auth-id' not in auth_params


@patch('cloudns_api.api.CLOUDNS_API_SUB_AUTH_USER', new='sub-user')
@patch('cloudns_api.api.CLOUDNS_API_AUTH_ID', new=None)
def test_get_auth_params_returns_sub_auth_user():
    """Function get_auth_params() returns sub auth params."""
    auth_params = get_auth_params()
    assert auth_params['sub-auth-user'] == 'sub-user'
    assert 'auth-id' not in auth_params


def test_get_auth_params_returns_different_dict_every_time():
    """Function get_auth_params returns a different dict that is not modified
    from previous operations."""
    auth_params = get_auth_params()
    auth_params['a-param'] = 'a-value'
    assert 'a-param' in auth_params

    new_params = get_auth_params()
    assert 'a-param' not in new_params


##
#  SnakeCase Tests

def test_snake_case_converts_keys_to_snake_case():
    pre_normalized_dict = {
        'test':          123,
        'testTest':      123,
        'testTestTest':  123,
        'testTTL':       123,
    }

    normalized_dict = {
        'test':            123,
        'test_test':       123,
        'test_test_test':  123,
        'test_ttl':        123,
    }

    assert use_snake_case_keys(pre_normalized_dict) == normalized_dict


##
#  ApiResponse Tests

def test_api_response_can_be_initialized_with_request_response():
    """An ApiResponse object can be initialized with a request response object.
    """
    request_response = RequestResponseStub(payload={'test': 123},
                                           status_code=200)
    response = ApiResponse(request_response)

    assert not response.error
    assert response.response == request_response
    assert response.success
    assert str(response.status_code) == '200'
    assert response.payload == {'test': 123}
    assert response.json() == {
            'status_code':   200,
            'success':      True,
            'payload':      {'test': 123}
        }


def test_api_response_works_with_request_response_list():
    """An ApiResponse object works with a request response object that is a
    list."""
    request_response = RequestResponseStub(payload=[{'test1': 123},
                                                    {'test2': 456},
                                                    {'test3': 789}],
                                           status_code=200)
    response = ApiResponse(request_response)

    assert response.payload == [{'test1': 123},
                                {'test2': 456},
                                {'test3': 789}]


def test_api_response_works_with_request_response_int():
    """An ApiResponse object works with a request response object that is
    simply an integer."""
    request_response = RequestResponseStub(payload=5, status_code=200)
    response = ApiResponse(request_response)

    assert response.payload == 5


@set_no_debug
def test_api_response_can_be_initialized_without_request_response():
    """An ApiResponse object can be initialized without a request response
    object."""
    response = ApiResponse()

    assert not response.error
    assert not response.response
    assert not response.success
    assert not response.status_code
    assert response.payload == {}
    assert response.json() == {
            'status_code':  None,
            'success':      False,
            'payload':      {}
        }


@set_debug
def test_api_response_without_request_response_shows_error_when_debugging():
    """An ApiResponse object that is initialized without a request response
    object shows an error in the json when debugging."""
    response = ApiResponse()

    # It's not yet initilized with an error
    assert response.error is None

    # But it will give an error converting to json (when debug mode is set)
    assert response.json()['error'] == \
        'Response has not yet been created with a requests.response.'


def test_api_response_can_have_error_set_on_response_wout_request_response():
    """An ApiResponse object can have an error set on a response that has been
    initilized without a request response."""
    response = ApiResponse()

    response.error = 'Test error'

    assert response.error == 'Test error'
    assert response.response is None
    assert not response.success
    assert not response.status_code
    assert response.payload == {}


def test_api_response_can_be_created_with_request_response_after_init():
    """ApiResponse can be created with request response after it is
    initialized."""
    request_response = RequestResponseStub(payload={'test': 123},
                                           status_code=200)
    response = ApiResponse()
    response.create(request_response)

    assert not response.error
    assert response.response == request_response
    assert response.success
    assert str(response.status_code) == '200'
    assert response.payload == {'test': 123}
    assert response.json() == {
            'status_code':  200,
            'success':      True,
            'payload':      {'test': 123}
        }


def test_api_response_can_be_converted_to_string():
    """ApiResponse can be converted to a string."""
    request_response = RequestResponseStub(payload={'test': 123},
                                           status_code=200)
    response = ApiResponse(request_response)

    expected_string = '{"status_code": 200, "success": true, "payload": ' \
        + '{"test": 123}}'

    assert response.string() == expected_string
    assert str(response) == expected_string


def test_api_response_payload_is_normalized_to_snake_case():
    """An ApiResponse object's payload keys are normalized to snake case."""
    pre_normalized_data = {
        'test':          123,
        'testTest':      123,
        'testTestTest':  123,
        'testTTL':       123,
    }
    request_response = RequestResponseStub(payload=pre_normalized_data)
    response = ApiResponse(request_response)

    assert response.payload == {
        'test':            123,
        'test_test':       123,
        'test_test_test':  123,
        'test_ttl':        123,
    }


def test_request_response_has_status_code():
    """An RequestResponseStub object's has expected properties."""
    response = RequestResponseStub(status_code=301)

    assert str(response.status_code) == '301'

    assert str(RequestResponseStub().status_code) == '200'


def test_request_response_has_a_payload():
    """An RequestResponseStub object's has expected properties."""
    response = RequestResponseStub(payload={'abc': 123, 'def': 456})

    assert response.payload == {'abc': 123, 'def': 456}
    assert response.json() == {'abc': 123, 'def': 456}


def test_request_response_can_be_used_for_tests():
    """An RequestResponseStub object's has expected properties."""
    response = RequestResponseStub(payload={
                                   'url': 'https://example.com/',
                                   'params': {'abc': 123, 'def': 456}})

    assert response.json() == {'url': 'https://example.com/',
                               'params': {'abc': 123, 'def': 456}}


##
# API Decorator Tests

def test_api_decorator_responds_to_success():
    """API decorator responds appropriately to successful requests."""

    @api
    def test_api_call(*args, **kwargs):
        return RequestResponseStub(payload={'response': 'Testing...'})

    response = test_api_call()
    assert response.success


def test_api_decorator_responds_to_network_errors():
    """API decorator responds appropriately to network errors."""

    @api
    def test_api_call(*args, **kwargs):
        raise request_exceptions.ConnectionError()

    response = test_api_call()
    assert response.success is False
    assert response.error == 'API Network Connection error.'
    assert str(response.status_code) == '500'


def test_api_decorator_responds_to_timeout_errors():
    """API decorator responds appropriately to network timeout errors."""

    @api
    def test_api_call(*args, **kwargs):
        raise request_exceptions.ConnectTimeout()

    response = test_api_call()
    assert not response.success
    assert response.error == 'API Connection timed out.'
    assert str(response.status_code) == '504'


def test_api_decorator_responds_to_500_errors():
    """API decorator responds appropriately to non-200 status codes."""

    @api
    def test_api_call(*args, **kwargs):
        return RequestResponseStub(payload={'response': 'Testing...'},
                                   status_code='500')

    response = test_api_call()
    assert not response.success
    assert str(response.status_code) == '500'


@set_no_debug
def test_api_decorator_responds_to_bad_python_code():
    """API decorator responds appropriately to bad python code."""

    @api
    def test_api_call(*args, **kwargs):
        # Bad python code:
        return uninitialized_variable  # noqa: F821

    response = test_api_call()
    assert not response.success
    assert response.error == 'Something went wrong.'
    assert str(response.status_code) == '500'


@set_debug
def test_api_decorator_responds_specifically_to_bad_code_when_debugging():
    """API decorator responds more specifically to bad python code when in
    debug mode."""

    @api
    def test_api_call(*args, **kwargs):
        # Bad python code:
        return uninitialized_variable  # noqa: F821

    response = test_api_call()
    assert not response.success
    assert response.error == "name 'uninitialized_variable' is not defined"


@set_no_debug
def test_api_decorator_responds_to_missing_required_args():
    """API decorator responds appropriately to required missing args."""

    @api
    def test_api_call(required_arg, *args, **kwargs):
        return uninitialized_variable  # noqa: F821

    response = test_api_call()
    assert not response.success
    assert response.error == 'Something went wrong.'
    assert str(response.status_code) == '500'


def test_api_decorator_responds_to_api_exceptions():
    """API decorator responds appropriately to more generic API exceptions."""

    @api
    def test_api_call(*args, **kwargs):
        raise ApiException('There was an error.')

    response = test_api_call()
    assert not response.success
    assert response.error == 'There was an error.'
    assert str(response.status_code) == '400'


def test_api_decorator_responds_to_derived_api_exceptions():
    """API decorator responds appropriately to derived API exceptions."""

    class ChildException(ApiException):
        status_code = '403'

    @api
    def test_api_call(*args, **kwargs):
        raise ChildException('There was an error.')

    response = test_api_call()
    assert not response.success
    assert response.error == 'There was an error.'
    assert str(response.status_code) == '403'


def test_api_decorator_responds_to_validation_error():
    """API decorator responds appropriately to validation error."""

    @api
    def test_api_call(*args, **kwargs):
        raise ValidationError('the_field_name',
                              'The field should be in this format.')

    response = test_api_call()
    assert not response.success
    assert response.error == 'Validation error.'
    assert response.validation_errors[0]['fieldname'] == 'the_field_name'
    assert response.validation_errors[0]['message'] == \
        'The field should be in this format.'


def test_api_decorator_responds_to_authentication_error():
    """API decorator responds appropriately to authentication error."""

    @api
    def test_api_call(*args, **kwargs):
        return RequestResponseStub(payload={
                                   'status': 'Failed',
                                   'statusDescription':
                                   'Invalid authentication, incorrect ' +
                                   'auth-id or auth-password.'})

    response = test_api_call()
    assert not response.success
    assert response.error == \
        'Invalid authentication, incorrect auth-id or auth-password.'


def test_api_patch_update_decorator_gets_then_updates():
    """API patch_update decorator gets before updating allowing patch updates
    with only some of the arguments."""

    @api
    def test_api_get(*args, **kwargs):
        assert len(kwargs) == 1
        assert kwargs['domain_name'] == 'my_example.com'
        return RequestResponseStub(payload={'key_1': 'AAA', 'key_2': 'BBB',
                                            'key_3': 'CCC', 'key_4': 'DDD'})

    @api
    @patch_update(get=test_api_get, keys=['domain_name'])
    def update(*args, **kwargs):
        return RequestResponseStub(payload=kwargs)

    response = update(domain_name='my_example.com', key_3='ZZZ', key_4='YYY',
                      patch=True)

    assert response.success
    assert response.payload['key_1'] == 'AAA'
    assert response.payload['key_2'] == 'BBB'
    assert response.payload['key_3'] == 'ZZZ'
    assert response.payload['key_4'] == 'YYY'


def test_api_patch_update_decorator_works_with_2_get_keys():
    """API patch_update decorator works with 2 get keys."""

    @api
    def test_api_get(*args, **kwargs):
        assert len(kwargs) == 2
        assert kwargs['domain_name'] == 'my_example.com'
        assert kwargs['id'] == 123
        return RequestResponseStub(payload={'key_1': 'AAA', 'key_2': 'BBB',
                                            'key_3': 'CCC', 'key_4': 'DDD'})

    @api
    @patch_update(get=test_api_get, keys=['domain_name', 'id'])
    def update(*args, **kwargs):
        return RequestResponseStub(payload=kwargs)

    response = update(domain_name='my_example.com', id=123, key_3='ZZZ',
                      key_4='YYY', patch=True)

    assert response.success
    assert response.payload['key_1'] == 'AAA'
    assert response.payload['key_2'] == 'BBB'
    assert response.payload['key_3'] == 'ZZZ'
    assert response.payload['key_4'] == 'YYY'


def test_api_patch_update_fails_when_get_first_fails():
    """API patch_update call fails when the get request first fails."""

    @api
    def test_api_get(*args, **kwargs):
        return RequestResponseStub(payload={'status': 'Failed',
                                            'statusDescription':
                                            'Missing domain-name'})

    @api
    @patch_update(get=test_api_get, keys=['domain_name'])
    def update(*args, **kwargs):
        return RequestResponseStub(payload=kwargs)

    response = update(domain_name='my_example.com', key_3='ZZZ', key_4='YYY',
                      patch=True)

    assert not response.success
    assert response.error == 'Missing domain-name'


@use_test_auth
@mock_get_request()
def test_simple_api_functions(test_id, test_password):
    """Tests that get_login, get_nameservers, and get_my_ip send properly
    formated requests."""
    expected_payload = {
        'auth-id': test_id,
        'auth-password': test_password,
    }

    response = get_login()
    assert response.payload['params'] == expected_payload

    response = get_nameservers()
    assert response.payload['params'] == expected_payload

    response = get_my_ip()
    assert response.payload['params'] == expected_payload



# -*- coding: utf-8 -*-
#
# name:             test_parameters.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       05/26/2019
#

"""
Functional tests for cloudns_api's parameters utilities module.
"""

from pytest import raises

from cloudns_api.parameters import Parameters
from cloudns_api.validation import ValidationError, ValidationErrorsBatch

from .helpers import use_test_auth


##
# Parameters object Tests

@use_test_auth
def test_parameters_includes_authentication_variables(test_id, test_password):
    """Parameters object includes authentication variables by default."""
    params = Parameters({}, validate=False)

    assert params._params_with_options['auth-id'] == test_id
    assert params._params_with_options['auth-password'] == test_password


@use_test_auth
def test_parameters_authentication_variables_can_be_overriden(test_id,
                                                              test_password):
    """Parameters object can override default authentication variables during
    initialization."""
    params_1 = Parameters({'auth-id': 'changed_id'}, validate=False)

    assert params_1._params_with_options['auth-id'] == 'changed_id'
    assert params_1._params_with_options['auth-password'] == test_password

    params_2 = Parameters({'auth-password': 'changed_password'},
                          validate=False)

    assert params_2._params_with_options['auth-id'] == test_id
    assert params_2._params_with_options['auth-password'] == 'changed_password'


@use_test_auth
def test_parameters_can_be_converted_to_dict(test_id, test_password):
    """Parameters object can be converted to a dict of parameter name and
    paramater value."""
    params = Parameters({
        'domain-name': {
            'value':    'example.com',
            'optional': False,
        },
        'host': {
            'value':    '@',
            'optional': False,
        },
        'type': 'TXT',
    }, validate=False)

    assert params.to_dict() == {
        'auth-id':        test_id,
        'auth-password':  test_password,
        'domain-name':    'example.com',
        'host':           '@',
        'type':           'TXT',
    }


@use_test_auth
def test_parameters_arent_affected_by_validation(test_id, test_password):
    """Parameters object can be converted to a dict of parameter name and
    paramater value even after validation."""
    params = Parameters({
        'domain-name':  'example.com',
        'primary-ns':    'ns1.example.com',
        'admin-mail':    'test@example.com',
        'ttl':           60,
        'refresh':  {
            'value':  1200,
            'min_value':  1200,
            'max_value': 43200
        },
        'retry':  {
            'value':  180,
            'min_value':  180,
            'max_value':  2419200
        },
        'expire':  {
            'value': 1209600,
            'min_value': 1209600,
            'max_value': 2419200
        },
        'default-ttl':  {
            'value':  60,
            'min_value':  60,
            'max_value':  2419200
        }
    }, validate=True)

    assert params.to_dict() == {
        'auth-id':        test_id,
        'auth-password':  test_password,
        'domain-name':    'example.com',
        'ttl':            60,
        'primary-ns':     'ns1.example.com',
        'admin-mail':     'test@example.com',
        'refresh':        1200,
        'retry':          180,
        'expire':         1209600,
        'default-ttl':    60,
    }


def test_parameters_can_be_validated_with_validation_options():
    """Parameters object can be validated."""
    params = Parameters({
        'domain-name':  {
            'value':  '',
            'optional':  False,
        },
    }, validate=False)

    with raises(ValidationError):
        params.validate()


def test_parameters_can_be_validated_without_validation_options():
    """Parameters object can be validated without validation options."""
    params = Parameters({'domain-name':  ''}, validate=False)

    with raises(ValidationError):
        params.validate()


def test_parameters_can_catch_multiple_validation_errors():
    """Parameters object can validate multiple parameters at one time."""
    params = Parameters({
        'domain-name':  {
            'value':  '',
            'optional':  False,
        },
        'host':  {
            'value':  '',
            'optional':  False,
        },
    }, validate=False)

    with raises(ValidationErrorsBatch):
        params.validate()


def test_parameters_are_validated_on_init_by_default():
    """Parameters object can be validated on instatiation."""
    with raises(ValidationError):
        Parameters({
            'domain-name':  {
                'value':  '',
                'optional':  False,
            },
        })



# -*- coding: utf-8 -*-
#
# name:             test_validation.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/18/2018
#

"""
Functional tests for cloudns_api's validation module.
"""

from pytest import raises

from cloudns_api.validation import (
    is_algorithm,
    is_api_bool,
    is_caa_flag,
    is_caa_type,
    is_domain_name,
    is_email,
    is_fptype,
    is_int,
    is_ipv4,
    is_ipv6,
    is_record_type,
    is_redirect_type,
    is_required,
    is_rows_per_page,
    is_tlsa_matching_type,
    is_tlsa_selector,
    is_tlsa_usage,
    is_ttl,
    is_valid,
    is_zone_type,
    validate,
    ValidationError,
    ValidationErrorsBatch,
)


##
# Validation Exception Tests

def test_validation_error_exception_can_return_error_details():
    """Tests ValidationError exception."""
    validation_error = ValidationError('name-of-field', 'The error message.')

    details = validation_error.get_details()
    assert details[0]['fieldname'] == 'name-of-field'
    assert details[0]['message'] == 'The error message.'


def test_validation_errors_batch_returns_all_error_details():
    """Tests ValidationErrorsBatch exception."""
    error_1 = ValidationError('first-field', 'The first error message.')
    error_2 = ValidationError('second-field', 'The second error message.')
    error_3 = ValidationError('third-field', 'The third error message.')

    errors = [error_1, error_2, error_3]

    validation_errors_batch = ValidationErrorsBatch(errors)
    details = validation_errors_batch.get_details()

    assert details[0]['fieldname'] == 'first-field'
    assert details[0]['message'] == 'The first error message.'
    assert details[1]['fieldname'] == 'second-field'
    assert details[1]['message'] == 'The second error message.'
    assert details[2]['fieldname'] == 'third-field'
    assert details[2]['message'] == 'The third error message.'


##
# Validate Function Tests

def test_validate_function_uses_validation_functions_dict():
    """Function validate() uses validation_functions dict to validate
    values."""
    integer = 1234
    domain = 'example.com'

    assert validate('integer', integer)
    assert validate('domain', domain)

    with raises(ValidationError):
        validate('ttl', domain)


def test_validate_function_allows_for_optional_fields():
    """Function validate() allows for optional fields."""
    assert validate('domain', None, optional=True)
    assert validate('domain', '', optional=True)

    with raises(ValidationError):
        validate('domain', '')

    with raises(ValidationError):
        validate('domain', None)

    # But this should not raise an error
    assert validate('domain', 0)


def test_validate_function_still_checks_given_optional_fields():
    """Function validate() validates optional fields if they are given."""
    optional_int = 'abcd'

    with raises(ValidationError):
        validate('ttl', optional_int)


def test_validate_function_accepts_validate_as_argument():
    """Function validate() accepts a  of additional validators to run."""
    int_value = 123
    not_int_value = 'abc'

    assert validate('new_field', int_value, validate_as='integer')

    with raises(ValidationError):
        assert validate('new_field', not_int_value, validate_as='integer')


##
# Specific Validation Functions Tests

def test_is_algorithm_validates_correctly():
    """Function is_algorithm() validates appropriately."""
    is_an_algorithm = 1
    also_is_an_algoritm = 'rsa'
    not_algorithm = 'vcr'
    also_not_algorithm = 10

    assert is_algorithm(is_an_algorithm, 'algorithm')
    assert is_algorithm(also_is_an_algoritm, 'algorithm')

    with raises(ValidationError):
        is_algorithm(not_algorithm, 'algorithm')

    with raises(ValidationError):
        is_algorithm(also_not_algorithm, 'algorithm')


def test_is_api_bool_validates_correctly():
    """Function is_api_bool() validates if any value is provided."""
    is_bool = 0
    also_bool = 1

    not_bool = 8701
    also_not_bool = 'true'

    assert is_api_bool(is_bool, 'test_bool')
    assert is_api_bool(also_bool, 'test_bool')

    with raises(ValidationError):
        is_api_bool(not_bool, 'test_bool')

    with raises(ValidationError):
        is_api_bool(also_not_bool, 'test_bool')


def test_is_caa_flag_validates_correctly():
    """Function is_caa_flag() validates true for 0 and 128."""
    is_flag = 0
    also_flag = 128
    not_flag = 1

    assert is_caa_flag(is_flag, 'caa_flag')
    assert is_caa_flag(also_flag, 'caa_flag')

    with raises(ValidationError):
        is_caa_flag(not_flag, 'caa_flag')


def test_is_caa_type_validates_correctly():
    """Function is_caa_type() validates true for 0 and 128."""
    is_type = 'issuewild'
    not_type = 'abc'

    assert is_caa_type(is_type, 'caa_type')

    with raises(ValidationError):
        is_caa_type(not_type, 'caa_type')


def test_is_domain_name_validates_domain_names():
    """Function is_domain_name() validates if a value is a valid domain
    name."""
    domain_name = 'example.com'
    also_a_domain_name = 'sub.example.com'

    not_a_domain_name = 'example@gmail.com'
    also_not_a_domain_name = 'http://example.com/'

    assert is_domain_name(domain_name, 'test_domain_field')
    assert is_domain_name(also_a_domain_name, 'test_domain_field')

    with raises(ValidationError) as exception:
        is_domain_name(not_a_domain_name, 'test_domain_field')

    assert exception.value.details['fieldname'] == 'test_domain_field'
    assert exception.value.details['message'] == \
        'This field must be a valid domain name.'

    with raises(ValidationError) as exception:
        is_domain_name(also_not_a_domain_name, 'test_domain_field')


def test_is_email_validates_emails():
    """Function is_email() validates if a value is a valid email."""
    email = 'test@example.com'
    also_an_email = 'test+spam@example.com'

    not_an_email = 'example@@com'
    also_not_an_email = 'http://example.com/'

    assert is_email(email, 'test_email')
    assert is_email(also_an_email, 'test_email')

    with raises(ValidationError) as exception:
        is_email(not_an_email, 'test_email')

    assert exception.value.details['fieldname'] == 'test_email'
    assert exception.value.details['message'] == \
        'This field must be a valid email.'

    with raises(ValidationError) as exception:
        is_email(also_not_an_email, 'test_email')


def test_is_fptype_validates_correctly():
    """Function is_fptype() validates appropriately."""
    is_an_fptype = 2
    also_an_fptype = 'Sha-256'
    not_an_fptype = 100
    also_not_an_fptype = 'sha-3'

    assert is_fptype(is_an_fptype, 'fptype')
    assert is_fptype(also_an_fptype, 'fptype')

    with raises(ValidationError):
        is_fptype(not_an_fptype, 'fptype')

    with raises(ValidationError):
        is_fptype(also_not_an_fptype, 'fptype')


def test_is_int_validates_integer_values():
    """Function is_int() validates if a value is an integer or not."""
    integer = 1
    also_integer = '1'
    not_integer = '1abc'
    also_not_integer = 'abc'

    assert is_int(integer, 'test_field')
    assert is_int(also_integer, 'test_field')

    with raises(ValidationError) as exception:
        is_int(not_integer, 'test_field')

    assert exception.value.details['fieldname'] == 'test_field'
    assert exception.value.details['message'] == \
        'This field must be an integer.'

    with raises(ValidationError):
        is_int(also_not_integer, 'test_field')


def test_is_int_validates_max_range():
    """Function is_int() validates if a value is less than max range."""
    just_right = 15
    too_big = 25

    assert is_int(just_right, 'test_field', min_value=10, max_value=20)

    with raises(ValidationError) as exception:
        is_int(too_big, 'test_field', min_value=10, max_value=20)

    assert exception.value.details['fieldname'] == 'test_field'
    assert exception.value.details['message'] == \
        'This field must be less than 20.'


def test_is_int_validates_min_range():
    """Function is_int() validates if a value is more than a min range."""
    just_right = 15
    too_small = 1

    assert is_int(just_right, 'test_field', min_value=10)

    with raises(ValidationError) as exception:
        is_int(too_small, 'test_field', min_value=10)

    assert exception.value.details['fieldname'] == 'test_field'
    assert exception.value.details['message'] == \
        'This field must be greater than 10.'


def test_is_ipv4_validates_ipv4_addresses():
    """Function is_ipv4() validates IPv4 addresses."""
    ip = '10.0.0.1'
    also_ip = '8.8.8.8'
    not_ip = '8.8.8.8.8'
    also_not_ip = '12345678abcdef'

    assert is_ipv4(ip)
    assert is_ipv4(also_ip)

    with raises(ValidationError):
        is_ipv4(not_ip)

    with raises(ValidationError):
        is_ipv4(also_not_ip)


def test_is_ipv6_validates_ipv6_addresses():
    """Function is_ipv6() validates IPv6 addresses."""
    ip = '2009:0db8:95a3:0000:0000:9a2e:0370:8334'
    also_ip = '2009:db8:95a3:0:0:9a2e:370:8334'
    not_ip = '8.8.8.8'
    also_not_ip = '1234:5678:abcdef'
    again_also_not_ip = '2009:0db8:95a3:0000:10000:9a2e:0370:8334'

    assert is_ipv6(ip)
    assert is_ipv6(also_ip)

    with raises(ValidationError):
        is_ipv6(not_ip)

    with raises(ValidationError):
        is_ipv6(also_not_ip)

    with raises(ValidationError):
        is_ipv6(again_also_not_ip)


def test_is_record_type_validates_types():
    """Function is_record_type() validates if a value is a valid domain record
    type."""
    is_type = 'Mx'
    also_type = 'aaaa'

    not_type = 'XM'
    also_not_type = 123

    assert is_record_type(is_type, 'test_type')
    assert is_record_type(also_type, 'test_type')

    with raises(ValidationError) as exception:
        is_record_type(not_type, 'test_type')

    assert exception.value.details['fieldname'] == 'test_type'
    assert exception.value.details['message'] == \
        'This field must be a valid domain record type.'

    with raises(ValidationError) as exception:
        is_record_type(also_not_type, 'test_type')


def test_is_redirect_type_validates_correctly():
    """Function is_redirect_type() validates true for 301 and 302."""
    is_redirect = 301
    also_redirect = 302
    not_redirect = 1

    assert is_redirect_type(is_redirect, 'redirect')
    assert is_redirect_type(also_redirect, 'redirect')

    with raises(ValidationError):
        is_redirect_type(not_redirect, 'redirect')


def test_is_required_validates_correctly():
    """Function is_required() validates if any value is provided."""
    value = 'the value'
    also_value = 0
    no_value = ''
    also_no_value = None

    assert is_required(value, 'required')
    assert is_required(also_value, 'required')

    with raises(ValidationError):
        is_required(no_value, 'required')

    with raises(ValidationError):
        is_required(also_no_value, 'required')


def test_is_rows_per_page_validates_correctly():
    """Function is_rows_per_page() validates correctly."""
    valid = 10
    also_valid = '50'
    not_valid = 11
    also_not_valid = '51'

    assert is_rows_per_page(valid, 'rows-per-page')
    assert is_rows_per_page(also_valid, 'rows-per-page')

    with raises(ValidationError):
        is_rows_per_page(not_valid, 'rows-per-page')

    with raises(ValidationError):
        is_rows_per_page(also_not_valid, 'rows-per-page')


def test_is_tlsa_matching_type_validates_correctly():
    """Function is_rows_per_page() validates correctly."""
    valid = 1
    also_valid = '2'
    not_valid = 11
    also_not_valid = '51'

    assert is_tlsa_matching_type(valid, 'tlsa_matching_type')
    assert is_tlsa_matching_type(also_valid, 'tlsa_matching_type')

    with raises(ValidationError):
        is_tlsa_matching_type(not_valid, 'tlsa_matching_type')

    with raises(ValidationError):
        is_tlsa_matching_type(also_not_valid, 'tlsa_matching_type')


def test_is_tlsa_selector_validates_correctly():
    """Function is_rows_per_page() validates correctly."""
    valid = 1
    also_valid = '0'
    not_valid = 2
    also_not_valid = '3'

    assert is_tlsa_selector(valid, 'tlsa_selector')
    assert is_tlsa_selector(also_valid, 'tlsa_selector')

    with raises(ValidationError):
        is_tlsa_selector(not_valid, 'tlsa_selector')

    with raises(ValidationError):
        is_tlsa_selector(also_not_valid, 'tlsa_selector')


def test_is_tlsa_usage_validates_correctly():
    """Function is_rows_per_page() validates correctly."""
    valid = 1
    also_valid = '3'
    not_valid = 4
    also_not_valid = '51'

    assert is_tlsa_usage(valid, 'tlsa_usage')
    assert is_tlsa_usage(also_valid, 'tlsa_usage')

    with raises(ValidationError):
        is_tlsa_usage(not_valid, 'tlsa_usage')

    with raises(ValidationError):
        is_tlsa_usage(also_not_valid, 'tlsa_usage')


def test_is_ttl_validates_cloudns_ttls():
    """Function is_ttl() validates if a value is a valid ClouDNS ttl."""
    ttl = 86400
    also_ttl = '12 Hours'
    again_also_ttl = '86400'

    not_ttl = 8701
    also_not_ttl = '14 Hours'

    assert is_ttl(ttl, 'test_ttl')
    assert is_ttl(also_ttl, 'test_ttl')
    assert is_ttl(again_also_ttl, 'test_ttl')

    with raises(ValidationError):
        is_ttl(not_ttl, 'test_ttl')

    with raises(ValidationError):
        is_ttl(also_not_ttl, 'test_ttl')


def test_is_valid_always_returns_true():
    """Function is_valid() always returns true."""
    assert is_valid(None, 'test')
    assert is_valid(123, 'test')
    assert is_valid('abc', 'test')


def test_is_zone_type_validates_correctly():
    """Function is_zone_type() validates if a value is a valid ClouDNS zone
    type."""
    zone_type = 'Master'
    also_zone_type = 'slave'

    not_zone_type = 123
    also_not_zone_type = 'masterdns'

    assert is_zone_type(zone_type, 'test_zone_type')
    assert is_zone_type(also_zone_type, 'test_zone_type')

    with raises(ValidationError):
        is_zone_type(not_zone_type, 'test_zone_type')

    with raises(ValidationError):
        is_zone_type(also_not_zone_type, 'test_zone_type')



# -*- coding: utf-8 -*-
#
# name:             zone.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/15/2018
#

"""
cloudns_api.zone
~~~~~~~~~~~~~~~~

This module contains API wrapper functions for listing, creating, updating,
getting, activating, and deleting zones.
"""

import requests

from .api import api
from .parameters import Parameters


@api
def list(page=1, rows_per_page=10, search='', group_id=''):
    """Returns a paginated list of zones

    :param page: (optional) int, current page
    :param rows_per_page: (optional) int, number of results on each page
    :param search: (optional) string used to search domain names, reverse zone
        name, or other keyword to search for in the zone names
    :param group_id: (optional) int, limits to zones within a group
    """
    url = 'https://api.cloudns.net/dns/list-zones.json'

    params = Parameters({
            'page':           page,
            'rows-per-page':  rows_per_page,
            'search': {
                'value':      search,
                'optional':   True,
            },
            'group-id': {
                'value':      group_id,
                'optional':   True,
            },
        })

    return requests.get(url, params=params.to_dict())


@api
def get_page_count(rows_per_page=10, search='', group_id=''):
    """Returns the number of pages for the full listing or search listing

    :param rows_per_page: int, number of results on each page
    :param search: (optional) string used to search domain names, reverse zone
        name, or other keyword to search for in the zone names
    :param group_id: (optional) int, limits to zones within a group
    """
    url = 'https://api.cloudns.net/dns/get-pages-count.json'

    params = Parameters({
            'rows-per-page': rows_per_page,
            'search': {
                'value':     search,
                'optional':  True,
            },
            'group-id': {
                'value':      group_id,
                'optional':   True,
            },
        })

    return requests.get(url, params=params.to_dict())


@api
def create(domain_name=None, zone_type='master', ns=[], master_ip=None):
    """Creates a new DNS zone.

    :param domain_name: string, (required) the domain name for which to
        create the zone
    :param zone_type: string, (required) mastor, slave, parked, or geodns
    :param ns: list (optional) list of nameservers used for starting ns
        record; only for master domains
    :param master_ip: string (optional, required for slave domains) master
        server IP address
    """
    url = 'https://api.cloudns.net/dns/register.json'

    param_args = {
        'domain-name': domain_name,
        'zone-type':   zone_type,
    }

    if hasattr(zone_type, 'lower') and zone_type.lower() == 'slave':
        param_args['master-ip'] = {
            'value': master_ip,
            'optional': False,
        }

    if hasattr(zone_type, 'lower') and zone_type.lower() == 'master':
        param_args['ns'] = {
            'value':    ns,
            'optional': True,
        }

    params = Parameters(param_args)

    return requests.post(url, params=params.to_dict())


@api
def get(domain_name=None):
    """Retreives the DNS zone information for a particular domain.

    :param domain_name: string, (required) the domain name for which to
        retrieve the zone information
    """
    url = 'https://api.cloudns.net/dns/get-zone-info.json'

    params = Parameters({'domain-name': domain_name})

    return requests.get(url, params=params.to_dict())


@api
def update(domain_name=None):
    """Updates the DNS zone serial number.

    :param domain_name: string, (required) the domain name for which to
        update
    """
    url = 'https://api.cloudns.net/dns/update-zone.json'

    params = Parameters({'domain-name': domain_name})

    return requests.post(url, params=params.to_dict())


@api
def activate(domain_name=None):
    """Activates the domain's zone.

    :param domain_name: string, (required) the domain name to activate
    """
    url = 'https://api.cloudns.net/dns/change-status.json'

    params = Parameters({'domain-name': domain_name, 'status': 1})

    return requests.post(url, params=params.to_dict())


@api
def deactivate(domain_name=None):
    """Deactivates the domain's zone.

    :param domain_name: string, (required) the domain name to deactivate
    """
    url = 'https://api.cloudns.net/dns/change-status.json'

    params = Parameters({'domain-name': domain_name, 'status': 0})

    return requests.post(url, params=params.to_dict())


@api
def toggle_activation(domain_name=None):
    """Toggles the domain's zone activation.

    :param domain_name: string, (required) the domain name to toggle
    """
    url = 'https://api.cloudns.net/dns/change-status.json'

    params = Parameters({'domain-name': domain_name})

    return requests.post(url, params=params.to_dict())


@api
def delete(domain_name=None):
    """Deletes a domain's zone

    :param domain_name: string, (required) the domain name for the zone to
        delete
    """
    url = 'https://api.cloudns.net/dns/delete.json'

    params = Parameters({'domain-name': domain_name})

    return requests.post(url, params=params.to_dict())


@api
def get_stats():
    """Returns the total number of zones and limits according to your ClouDNS
    plan."""
    url = 'https://api.cloudns.net/dns/get-zones-stats.json'

    params = Parameters({})

    return requests.get(url, params=params.to_dict())


@api
def dnssec_available(domain_name=None):
    """Checks if DNSSEC is available for the domain name

    :param domain_name: string, (required) the domain name to check the DNSSEC
        availability for.
    """
    url = 'https://api.cloudns.net/dns/is-dnssec-available.json'

    params = Parameters({'domain-name': domain_name})

    return requests.get(url, params=params.to_dict())


@api
def dnssec_activate(domain_name=None):
    """Activate DNSSEC for the domain's zone

    :param domain_name: string, (required) the domain name to activate DNSSEC
        for.
    """
    url = 'https://api.cloudns.net/dns/activate-dnssec.json'

    params = Parameters({'domain-name': domain_name})

    return requests.post(url, params=params.to_dict())


@api
def dnssec_deactivate(domain_name=None):
    """Deactivate DNSSEC for the domain's zone

    :param domain_name: string, (required) the domain name to deactivate DNSSEC
        for.
    """
    url = 'https://api.cloudns.net/dns/deactivate-dnssec.json'

    params = Parameters({'domain-name': domain_name})

    return requests.post(url, params=params.to_dict())


@api
def dnssec_ds_records(domain_name=None):
    """Get the DNSSEC DS records for the domain's zone

    :param domain_name: string, (required) the domain name to retrieve the
        DNSSEC DS records for.
    """
    url = 'https://api.cloudns.net/dns/get-dnssec-ds-records.json'

    params = Parameters({'domain-name': domain_name})

    return requests.get(url, params=params.to_dict())

@api
def is_updated(domain_name=None):
    """Return True if dns zone is updatd on all servers and False if it is not

    :param domain_name: string, (required) the domain name to verify.
    """
    url = 'https://api.cloudns.net/dns/is-updated.json'

    params = Parameters({'domain-name': domain_name})

    return requests.get(url, params=params.to_dict())



# -*- coding: utf-8 -*-
#
# name:             record.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/15/2018
#

"""
cloudns_api.record
~~~~~~~~~~~~~~~~~~

This module contains API wrapper functions for DNS listing, creating, updating,
and deleting records.

There are also a functions for activating/deactivating a record, importing
records, copying existing records

For SOA records, see soa.py
"""

import requests

from .api import (
    ApiException,
    RequestResponseStub,
    api,
    get_auth_params,
    patch_update,
)
from .parameters import Parameters
from .validation import is_record_type, ValidationError


class RecordNotFound(ApiException):
    """Record not found API exception."""
    status_code = requests.codes.NOT_FOUND


@api
def get_available_record_types(zone_type):
    """Returns available record types for a zone

    :param zone_type: string, (required) the type of the zone (domain, reverse
        or parked)
    """
    url = 'https://api.cloudns.net/dns/get-available-record-types.json'

    params = Parameters({'zone-type': zone_type})

    return requests.get(url, params=params.to_dict())


@api
def get_available_ttls():
    """Returns a list of available ttls."""
    url = 'https://api.cloudns.net/dns/get-available-ttl.json'

    return requests.get(url, params=get_auth_params())


@api
def list(domain_name=None, host=None, record_type=''):
    """Lists DNS records for a particular domain.

    :param domain_name: string, the domain name for which to retrieve the
        records
    :param host: (optional) string, a host to limit results by. Use '@' for
        domain as host.
    :param record_type: (optional) string, the record type to retrieve (ie,
        'a', 'cname', etc..., See RECORD_TYPES)
    """
    url = 'https://api.cloudns.net/dns/records.json'

    params = Parameters({
            'domain-name':  domain_name,
            'host': {
                'value':    host,
                'optional': True
            },
            'type': {
                'value':    record_type,
                'optional': True
            },
        })

    return requests.get(url, params=params.to_dict())


def generate_dns_record_parameters(domain_name=None, record_type=None, host='',
                                   record=None, ttl=None,
                                   validate_record_as='valid', **kwargs):
    """Generates parameters for a generic DNS record.

    :param domain_name: string, (required) the domain name to use for the
        record
    :param record_type: string, the type of domain to create
    :param host: string, the host for this record. Use '' (empty string) for
        top-level domain
    :param record: string, (required) the record to be added or updated
    :param ttl: int, (required) the time-to-live for this record
    :param validate_record_as: string, what to validate the record parameter
        as
    """
    params = {
        'domain-name':     domain_name,
        'host': {
            'value':       host,
            'optional':    True
        },
        'ttl':             ttl,
        'record': {
            'value':       record,
            'validate_as': validate_record_as
        }
    }

    if record_type:
        params['record-type'] = record_type

    return params


def generate_a_record_parameters(**kwargs):
    """Generates parameters for 'A' records."""
    return generate_dns_record_parameters(record_type='A',
                                          validate_record_as='ipv4',
                                          **kwargs)


def generate_aaaa_record_parameters(**kwargs):
    """Generates parameters for 'AAAA' records."""
    return generate_dns_record_parameters(record_type='AAAA',
                                          validate_record_as='ipv6',
                                          **kwargs)


def generate_mx_record_parameters(priority=10, **kwargs):
    """Generates parameters for 'MX' records.

    :param priority: int, the priority for this server record
    """
    params = generate_dns_record_parameters(record_type='MX',
                                            validate_record_as='domain-name',
                                            **kwargs)
    params['priority'] = priority

    return params


def generate_cname_record_parameters(**kwargs):
    """Generates parameters for 'CNAME' records."""
    return generate_dns_record_parameters(record_type='CNAME',
                                          validate_record_as='domain-name',
                                          **kwargs)


def generate_txt_record_parameters(**kwargs):
    """Generates parameters for 'TXT' records."""
    return generate_dns_record_parameters(record_type='TXT',
                                          validate_record_as='valid',
                                          **kwargs)


def generate_spf_record_parameters(**kwargs):
    """Generates parameters for 'SPF' records."""
    return generate_dns_record_parameters(record_type='SPF',
                                          validate_record_as='valid',
                                          **kwargs)


def generate_ns_record_parameters(**kwargs):
    """Generates parameters for 'NS' records."""
    return generate_dns_record_parameters(record_type='NS',
                                          validate_record_as='domain-name',
                                          **kwargs)


def generate_srv_record_parameters(port=None, priority=None, weight=None,
                                   **kwargs):
    """Generates parameters for 'SRV' records.

    :param port: (optional) int, port for SRV record
    :param priority:  int, priority index, prioritize the lowest indexed server
    :param weight: int, a relative weight for services with the same index
    """
    params = generate_dns_record_parameters(record_type='SRV',
                                            validate_record_as='domain-name',
                                            **kwargs)
    if port:
        params['port'] = port

    params['priority'] = priority
    params['weight'] = weight

    return params


def generate_wr_record_parameters(redirect_type=None, frame=0,
                                  frame_title='', frame_keywords='',
                                  frame_description='', **kwargs):
    """Generates parameters for 'WR' records.

    :param redirect_type:  int, 301 (permanent) or 302 (temporary) redirect
    :param frame: int [0 or 1], redirect the url in a frame so it is
        "transparent" to the user. Use '1' to enable and '0' to disable.
    :param frame_title: string, title of the frame
    :param frame_keywords: string, keywords used in the frame
    :param frame_description: string, description of the frame
    """
    params = generate_dns_record_parameters(record_type='WR',
                                            validate_record_as='valid',
                                            **kwargs)
    params['redirect-type'] = redirect_type

    params['frame'] = frame
    params['frame-title'] = {'value': frame_title, 'optional': True}
    params['frame-description'] = {'value': frame_description,
                                   'optional': True}
    params['frame-keywords'] = {'value': frame_keywords, 'optional': True}

    return params


def generate_alias_record_parameters(**kwargs):
    """Generates parameters for 'ALIAS' records."""
    return generate_dns_record_parameters(record_type='ALIAS',
                                          validate_record_as='domain-name',
                                          **kwargs)


def generate_rp_record_parameters(**kwargs):
    """Generates parameters for 'RP' records."""
    return generate_dns_record_parameters(record_type='RP',
                                          validate_record_as='valid',
                                          **kwargs)


def generate_sshfp_record_parameters(algorithm=None, fptype=None, **kwargs):
    """Generates parameters for 'SSHFP' records.

    :param algorithm: string, (required) the algorithm the record uses ('RSA',
        'DSA', 'ECDSA', or 'ED25519')
    :param fptype: int, (required) the fingerprint type of the record (SHA-1 or
        SHA-256)
    """
    params = generate_dns_record_parameters(record_type='SSHFP',
                                            validate_record_as='valid',
                                            **kwargs)
    params['algorithm'] = algorithm
    params['fptype'] = fptype

    return params


def generate_ptr_record_parameters(**kwargs):
    """Generates parameters for 'PTR' records."""
    return generate_dns_record_parameters(record_type='PTR',
                                          validate_record_as='valid',
                                          **kwargs)


def generate_naptr_record_parameters(**kwargs):
    """Generates parameters for 'NAPTR' records."""
    return generate_dns_record_parameters(record_type='NAPTR',
                                          validate_record_as='valid',
                                          **kwargs)


def generate_caa_record_parameters(caa_flag=None, caa_type='', caa_value='',
                                   **kwargs):
    """
    :param caa_flag: int, 0 - Non critical or 128 - Critical
    :param caa_type: string, type of CAA record. The available flags are issue,
        issuewild, iodef.
    :param caa_value: type is issue, caa_value can be hostname or ";". If
        caa_type is issuewild, it can be hostname or ";".  If caa_type is
        iodef, it can be "mailto:someemail@address.tld, http://example.tld or
        http://example.tld.
    """
    params = generate_dns_record_parameters(record_type='SSHFP',
                                            validate_record_as='valid',
                                            **kwargs)
    params.pop('record')

    params['caa-flag'] = caa_flag
    params['caa-type'] = caa_type
    params['caa-value'] = caa_value

    return params


def generate_tlsa_record_parameters(tlsa_usage=0, tlsa_selector=0,
                                    tlsa_matching_type=0,
                                    **kwargs):
    """
    :param tlsa_usage: int, can take one of the following values:
        0 - PKIX-TA: Certificate Authority Constraint
        1 - PKIX-EE: Service Certificate Constraint
        2 - DANE-TA: Trust Anchor Assertion
        3 - DANE-EE: Domain Issued Certificate
    :param tlsa_selector: int, can take one of the following values:
        0 - Cert: Use full certificates
        1 - SPKI: Use subject public key
    :param tlsa_matching_type: int, can take one of the following values:
        0 - Full: No Hash
        1 - SHA-256: SHA-256 Hash
        2 - SHA-512: SHA-512 hash
    """
    params = generate_dns_record_parameters(record_type='TLSA',
                                            validate_record_as='valid',
                                            **kwargs)
    params.pop('record')

    params['tlsa_usage'] = caa_flag
    params['tlsa_selector'] = caa_type
    params['tlsa_matching_type'] = caa_value

    return params


generators = {
    'A':      generate_a_record_parameters,
    'AAAA':   generate_aaaa_record_parameters,
    'MX':     generate_mx_record_parameters,
    'CNAME':  generate_cname_record_parameters,
    'TXT':    generate_txt_record_parameters,
    'SPF':    generate_spf_record_parameters,
    'NS':     generate_ns_record_parameters,
    'SRV':    generate_srv_record_parameters,
    'WR':     generate_wr_record_parameters,
    'ALIAS':  generate_alias_record_parameters,
    'RP':     generate_rp_record_parameters,
    'SSHFP':  generate_sshfp_record_parameters,
    'PTR':    generate_ptr_record_parameters,
    'NAPTR':  generate_naptr_record_parameters,
    'CAA':    generate_caa_record_parameters,
    'TLSA':   generate_tlsa_record_parameters,
}


def generate_record_parameters(record_type=None, record_id=None, **kwargs):
    """Creates and returns DNS record Parameters object for create or update
    operations.

    This function (along with the generator functions) helps to avoid code
    duplication and allows each record type to have only the specific
    parameters necessary for that type.

    :param record_type: string, (required) the record type to retrieve (ie,
        'a', 'cname', etc..., See RECORD_TYPES)
    :param record_id: int, (required for updates) the record id
    """
    if not is_record_type(record_type):
        raise ValidationError('Not a valid domain record type')

    record_parameters = generators[record_type.upper()](**kwargs)

    if record_id:
        record_parameters['record-id'] = record_id

    return Parameters(record_parameters)


@api
def create(domain_name=None, record_type=None, **kwargs):
    """Creates a DNS record.

    :param domain_name: string, (required) the domain name for which to
        retrieve the records
    :param record_type: string, (required) the type of record to create

    Other parameters are required based on the type of record you are creating.
    See the generate_{record_type}_record_parameters function.
    """
    url = 'https://api.cloudns.net/dns/add-record.json'

    params = generate_record_parameters(domain_name=domain_name,
                                        record_type=record_type, **kwargs)

    return requests.post(url, params=params.to_dict())


@api
def transfer(domain_name=None, server=None):
    """Transfers all the domain records from one DNS server to ClouDNS's
    server. The domain name must be the same on both servers.

    :param domain_name: string, the domain name to transfer
    :param server: string, the server to transfer the records from (can be a
        domain name or ip address)
    """
    url = 'https://api.cloudns.net/dns/axfr-import.json'

    params = Parameters({'domain-name': domain_name, 'server': server})

    return requests.post(url, params=params.to_dict())


@api
def copy(domain_name=None, from_domain=None, delete_current_records=False):
    """Transfers all the domain records from one DNS server to ClouDNS's
    server. The domain name must be the same on both servers.

    :param domain_name: string, the domain name to copy to
    :param from_domain: string, the domain name from which to copy
    :param delete_current_records: int, whether or not to delete the current
        records.
    """
    url = 'https://api.cloudns.net/dns/copy-records.json'

    params = Parameters({
        'domain-name': domain_name,
        'from-domain': from_domain,
        'delete-current-records': 1 if delete_current_records else 0,
    })

    return requests.get(url, params=params.to_dict())


@api
def get(domain_name=None, record_id=None):
    """Returns a specific record_id for a domain.

    This is a wrapper around the list function extracting just the specified
    record.

    :param domain_name: string, the domain name
    :param record_id: int, the ClouDNS record id to return
    """
    response = list(domain_name)

    if str(record_id) not in response.payload:
        raise RecordNotFound('Record "' + str(record_id) + '" not found in "' +
                             domain_name + '" zone.')

    return RequestResponseStub(payload=response.payload[str(record_id)],
                               status_code=response.status_code)


@api
def export(domain_name=None):
    """Exports all of the records for a domain_name in BIND format.

    :param domain_name: string, the domain name to export
    """
    url = 'https://api.cloudns.net/dns/records-export.json'

    params = Parameters({'domain-name': domain_name})

    return requests.get(url, params=params.to_dict())


@api
def get_dynamic_url(domain_name=None, record_id=None):
    """Returns a url that is used to dynamically update an A or AAAA record to
    the IP address of the device that calls the URL.

    :param domain_name: string, the domain name
    :param record_id: int, the ClouDNS record id to return. Must be an A or
        AAAA record type.
    """
    url = 'https://api.cloudns.net/dns/get-dynamic-url.json'

    params = Parameters({'domain-name': domain_name, 'record-id': record_id})

    return requests.get(url, params=params.to_dict())


@api
@patch_update(get=get, keys=['domain_name', 'record_id'])
def update(domain_name=None, record_id=None, record_type=None, patch=False,
           **kwargs):
    """Updates a DNS record.

    Record types cannot be updated, but you can pass the existing record type
    to let the api know what kind of parameters to validate against. If you do
    not pass this argument, the API will make an extra call to retrieve it for
    you.

    :param domain_name: string, (required) the domain name for which to
        retrieve the records
    :param record_id: int, (required) the record id for the record to update
    :param patch: boolean, whether or not to do a patch update
    """
    url = 'https://api.cloudns.net/dns/mod-record.json'

    if not record_type:
        response = get(domain_name, record_id)

        if not response.success:
            return RequestResponseStub(payload=response.json(),
                                       status_code=response.status_code)

        record_type = response.payload['type']

    params = generate_record_parameters(domain_name=domain_name,
                                        record_type=record_type,
                                        record_id=record_id, **kwargs)

    # Record type should not be submitted in the request to ClouDNS
    params_as_dict = params.to_dict()
    params_as_dict.pop('record-type')

    return requests.post(url, params=params_as_dict)


def patch(*args, **kwargs):
    """A convenience function for patch updates."""
    return update(*args, patch=True, **kwargs)


@api
def activate(domain_name=None, record_id=None):
    """Makes a particular record on a domain name active

    :param domain_name: string, the domain name on which to work
    :param record_id: int, record id (the id returned when listing records)
    """
    url = 'https://api.cloudns.net/dns/change-record-status.json'

    params = Parameters({'domain-name': domain_name, 'record-id': record_id,
                         'status': 1})

    return requests.post(url, params=params.to_dict())


@api
def deactivate(domain_name=None, record_id=None):
    """Makes a particular record on a domain name inactive

    :param domain_name: string, the domain name on which to work
    :param record_id: int, record id (the id returned when listing records)
    """
    url = 'https://api.cloudns.net/dns/change-record-status.json'

    params = Parameters({'domain-name': domain_name, 'record-id': record_id,
                         'status': 0})

    return requests.post(url, params=params.to_dict())


@api
def toggle_activation(domain_name=None, record_id=None):
    """Toggles active/inactive status on a particular record of a domain name.

    :param domain_name: string, the domain name on which to work
    :param record_id: int, record id (the id returned when listing records)
    """
    url = 'https://api.cloudns.net/dns/change-record-status.json'

    params = Parameters({'domain-name': domain_name, 'record-id': record_id})

    return requests.post(url, params=params.to_dict())


@api
def delete(domain_name=None, record_id=None):
    """Deletes a DNS record

    :param domain_name: string, the domain name on which to work
    :param record_id: int, the specific record id of the record to delete
    """
    url = 'https://api.cloudns.net/dns/delete-record.json'

    params = Parameters({'domain-name': domain_name, 'record-id': record_id})

    return requests.post(url, params=params.to_dict())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pylint: disable=bad-whitespace,redefined-builtin

"""
cloudns_api setup
"""

import sys


if sys.version_info.major == 2:
    sys.exit('Sorry, Python 2 is not supported.')


from codecs import open
from setuptools import setup, find_packages

from cloudns_api import __version__ as version


with open('README.rst', 'r', 'utf-8') as f:
    README = f.read()

url = 'https://github.com/prestix-studio/cloudns_api/archive/v{}.tar.gz'\
        .format(version)

setup(
    name='cloudns_api',
    version=version,
    description='A python interface to the ClouDNS.net API',
    long_description=README,
    long_description_content_type='text/x-rst',
    url='https://github.com/prestix-studio/cloudns_api',
    download_url=url,
    author='Harold Bradley III | Prestix Studio, LLC',
    author_email='harold@prestix.studio',
    license='MIT License',
    keywords=['dns', 'dns api', 'domain name system', 'cloudns',
              'server development'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    packages=find_packages(),
    install_requires=['requests'],
    test_requires=['pytest>=3', 'mock'],
    package_data={'': ['LICENSE']},
    entry_points={},
)



# -*- coding: utf-8 -*-
#
# name:             config.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       05/27/2019
#

"""
cloudns_api.config
~~~~~~~~~~~~~~~~~~

This module contains the API config constants.
"""

from os import environ


def _is_true(env_var):
    if env_var is None:  # pragma: no cover
        return False
    return env_var.lower() in [1, 'true', 'yes']


CLOUDNS_API_TESTING = _is_true(environ.get('CLOUDNS_API_TESTING'))

CLOUDNS_API_AUTH_ID = environ.get('CLOUDNS_API_AUTH_ID')
CLOUDNS_API_SUB_AUTH_ID = environ.get('CLOUDNS_API_SUB_AUTH_ID')
CLOUDNS_API_SUB_AUTH_USER = environ.get('CLOUDNS_API_SUB_AUTH_USER')

CLOUDNS_API_AUTH_PASSWORD = environ.get('CLOUDNS_API_AUTH_PASSWORD')

CLOUDNS_API_DEBUG = _is_true(environ.get('CLOUDNS_API_DEBUG'))



# -*- coding: utf-8 -*-
#
# name:             helpers.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       05/26/2019
#

"""
Mock helpers for cloudns_api unit tests.
"""

from mock import patch

from cloudns_api.api import RequestResponseStub


##
# Config patches

TEST_ID = 'test-auth-id'
TEST_PASSWORD = 'test-auth-password'


def use_test_auth(test_fn):
    @patch('cloudns_api.api.CLOUDNS_API_AUTH_ID', new=TEST_ID)
    @patch('cloudns_api.api.CLOUDNS_API_AUTH_PASSWORD', new=TEST_PASSWORD)
    def test_wrapper(*args, **kwargs):
        test_fn(*args, test_id=TEST_ID, test_password=TEST_PASSWORD, **kwargs)
    return test_wrapper


def set_debug(test_fn):
    @patch('cloudns_api.api.CLOUDNS_API_DEBUG', new=True)
    def test_wrapper(*args, **kwargs):
        test_fn(*args, **kwargs)
    return test_wrapper


def set_no_debug(test_fn):
    @patch('cloudns_api.api.CLOUDNS_API_DEBUG', new=False)
    def test_wrapper(*args, **kwargs):
        test_fn(*args, **kwargs)
    return test_wrapper


##
# Request Mock functions

def mock_get_request(payload=None):
    """A closure decorator to pass payload for the get_mock function.

    Note: You must use parens even when you pass no arguments.
        @mock_get_request()
    """
    def get_mock(url, params=None, payload=payload):
        """A mock get request to return the request-prepared url."""
        if not payload:
            payload = {
                'url': url,
                'params': params,
            }
        return RequestResponseStub(payload=payload)

    def decorator(test_fn):
        """Having the outer decorator allows passing arguments. This inner
        decorator is where the function is passed."""
        @patch('cloudns_api.requests.get', new=get_mock)
        def test_wrapper(*args, **kwargs):
            test_fn(*args, **kwargs)
        return test_wrapper
    return decorator


def mock_post_request(payload=None):
    """A closure decorator to pass payload for the post_mock function.

    Note: You must use parens even when you pass no arguments.
        @mock_post_request()
    """
    def post_mock(url, params=None, payload=payload):
        """A mock post request to return the request-prepared url."""
        if not payload:
            payload = {
                'url': url,
                'params': params,
            }
        return RequestResponseStub(payload=payload)

    def decorator(test_fn):
        """Having the outer decorator allows passing arguments. This inner
        decorator is where the function is passed."""
        @patch('cloudns_api.requests.post', new=post_mock)
        def test_wrapper(*args, **kwargs):
            test_fn(*args, **kwargs)
        return test_wrapper
    return decorator



# -*- coding: utf-8 -*-
#
# name:             api.py
# author:           Harold Bradley III | Prestix Studio, LLC.
# email:            harold@prestix.studio
# created on:       09/15/2018
#

"""
cloudns_api.api
~~~~~~~~~~~~~~~

This module contains basic API utilities and the api decorator function.
"""

from json import dumps as to_json_string
from re import compile as re_compile
import requests
from requests import codes as code
from requests.exceptions import (
    ContentDecodingError,
    ConnectionError,
    HTTPError,
    SSLError,
    TooManyRedirects,
    ConnectTimeout,
    Timeout,
    ReadTimeout,
)

from .config import (
    CLOUDNS_API_AUTH_ID,
    CLOUDNS_API_AUTH_PASSWORD,
    CLOUDNS_API_DEBUG,
    CLOUDNS_API_SUB_AUTH_ID,
    CLOUDNS_API_SUB_AUTH_USER,
    CLOUDNS_API_TESTING,
)
from .validation import ValidationError


def get_auth_params():
    """Returns a dict pre-populated with auth parameters."""
    # Get password parameter
    if not CLOUDNS_API_TESTING and not CLOUDNS_API_AUTH_PASSWORD:  # pragma: no cover
        raise EnvironmentError(
            'Environment variable "CLOUDNS_API_AUTH_PASSWORD" not set.'
        )
    auth_params = {'auth-password': CLOUDNS_API_AUTH_PASSWORD}

    # Get username parameter
    if CLOUDNS_API_AUTH_ID:
        auth_params['auth-id'] = CLOUDNS_API_AUTH_ID
    elif CLOUDNS_API_SUB_AUTH_ID:
        auth_params['sub-auth-id'] = CLOUDNS_API_SUB_AUTH_ID
    elif CLOUDNS_API_SUB_AUTH_USER:
        auth_params['sub-auth-user'] = CLOUDNS_API_SUB_AUTH_USER
    elif not CLOUDNS_API_TESTING:  # pragma: no cover
        raise EnvironmentError(
            'No environment variable "CLOUDNS_API_AUTH_ID", '
            '"CLOUDNS_API_SUB_AUTH_ID" or "CLOUDNS_API_SUB_AUTH_USER" is set.'
        )

    return auth_params


_first_cap_re = re_compile('(.)([A-Z][a-z]+)')
_all_cap_re = re_compile('([a-z0-9])([A-Z])')


def convert_to_snake_case(string):
    string = str(string)
    string = _first_cap_re.sub(r'\1_\2', string)
    string = _all_cap_re.sub(r'\1_\2', string).lower()
    return string


def use_snake_case_keys(original_dict):
    """Converts a dict's keys to snake case.

    @see https://stackoverflow.com/questions/1175208
    """
    normalized_dict = {}

    for key, value in original_dict.items():
        normalized_dict[convert_to_snake_case(key)] = value

    return normalized_dict


class ApiResponse(object):
    def __init__(self, response=None):
        """Wrapper object to add custom functionality and properties to a
        request response object.

        :param response: requests.models.response, Requests response object.
        """
        self.error = None
        self.validation_errors = None

        if response:
            self.create(response)
        else:
            self.response = None

    def __str__(self):
        """Returns the string value of the response."""
        return self.string()

    def create(self, response):
        """Creates the the response and checks for HTTP and API errors.

        :param response: requests.models.response, Requests response object.
        """
        self.response = response

        try:
            self.error = self.payload['error']
        except (TypeError, KeyError):
            pass

        # Check for HTTP error codes
        if not self.error and self.status_code is not code.OK:
            self.error = 'HTTP response ' + str(self.status_code)

        # Check for error responses from ClouDNS
        elif isinstance(self.payload, dict) \
                and 'status' in self.payload \
                and self.payload['status'] == 'Failed':
            self.error = self.payload['status_description']

    @property
    def success(self):
        """Returns the success status of the response. Successful when the
        status_code is ok and there are no error messages."""
        return self.status_code is code.OK and not self.error

    @property
    def status_code(self):
        """Wraps the request response's status code."""
        if hasattr(self, '_status_code'):
            return self._status_code
        return self.response.status_code if self.response else None

    @status_code.setter
    def status_code(self, status_code):
        """Status code setter"""
        self._status_code = status_code

    @property
    def payload(self):
        """Wraps the request response's json method."""
        if not self.response:
            return {}

        payload = self.response.json()  # Get the requests response json

        if isinstance(payload, dict):
            return use_snake_case_keys(payload)
        else:
            return payload

    def json(self):
        """Returns the response as a json object. This allows us to scrub the
        payload and massage it to our requirements."""
        json = {
            'status_code':  self.status_code,
            'success':      self.success,
            'payload':      self.payload,
        }

        if self.error:
            json['error'] = self.error

        if self.validation_errors:
            json['validation_errors'] = self.validation_errors

        if CLOUDNS_API_DEBUG and not self.success and not self.error:
            json['error'] = \
                'Response has not yet been created with a requests.response.'

        return json

    def string(self):
        """Returns the json response as a json string."""
        return to_json_string(self.json())


class RequestResponseStub(object):
    def __init__(self, payload=None, status_code=200):
        """Requests response stub used for stubbing request responses in the
        API code as well as for testing for testing.

        :param payload: dict, json data to be returned in the json() method.
        :param status_code: integer, the status code to return in the test.
            Defaults to 200.
        """
        self.payload = payload if payload else {}
        self.status_code = status_code

    def json(self):
        """Mocks the json object method."""
        return self.payload


class ApiException(Exception):
    status_code = code.BAD_REQUEST

    def __init__(self, message, *args, **kwargs):
        """An exception type reported and caught by the API.

        :param message: string a message to report to the user.
        """
        self.message = message
        super(ApiException, self).__init__(*args, **kwargs)


def api(api_call):
    """Decorates an api call in order to consistently handle errors and
    maintain a consistent json format.

    The decorated function should return a requests.response (or
    RequestResponseStub), and this wrapper will cause the function to finally
    return an ApiResponse.

    :param api_call: function, the function to be decorated
    """

    def api_wrapper(*args, **kwargs):
        """ Wraps an api call in order to consistently handle errors and
        maintain a consistent json format.
        """
        response = ApiResponse()

        try:
            response.create(api_call(*args, **kwargs))

        # Catch Timeout errors
        except (ConnectTimeout, Timeout, ReadTimeout) as e:
            response.error = 'API Connection timed out.'
            response.status_code = code.GATEWAY_TIMEOUT

            if CLOUDNS_API_DEBUG:  # pragma: no cover
                response.error = str(e)

        # Catch Connection errors
        except (ContentDecodingError, ConnectionError, HTTPError, SSLError,
                TooManyRedirects) as e:
            response.error = 'API Network Connection error.'
            response.status_code = code.SERVER_ERROR

            if CLOUDNS_API_DEBUG:  # pragma: no cover
                response.error = str(e)

        # Catch API reported exceptions
        except ApiException as e:
            response.error = e.message
            response.status_code = e.status_code

        # Catch Validation errors
        except ValidationError as e:
            response.error = 'Validation error.'
            response.validation_errors = e.get_details()
            response.status_code = code.BAD_REQUEST

        # Catch all other errors
        except Exception as e:
            response.error = 'Something went wrong.'
            response.status_code = code.SERVER_ERROR

            if CLOUDNS_API_DEBUG:
                response.error = str(e)

            if CLOUDNS_API_TESTING:
                import traceback
                print('\n' + traceback.format_exc())

        # Whew! Made it past the errors, so return the response
        return response

    return api_wrapper


def patch_update(get, keys):
    """Decorates an api call to allow 'patch' updating with only parameters to
    be updated.

    :param get: function, the function that should be used to get the existing
        parameters.
    :param keys: list, the list of keys that will be passed in the kwargs that
        should be used to get the existing data.
    """

    def decorated_patch_update(api_call):
        """Decorates an api call to allow 'patch' updating with only parameters to
        be updated.

        :param api_call: function, the function to be decorated
        """

        def api_wrapper(*args, **kwargs):
            """ Wraps an api call in order to allow 'patching'
            maintain a consistent json format.

            :param patch: bool, retrieve current parameter values to pass with
                the parameters given. If False, does nothing. False by default.
            """
            if 'patch' in kwargs and kwargs['patch']:
                response = get(*args, **{key: value for key, value in
                                         kwargs.items() if key in keys})
                if not response.success:
                    # Return the requests.response. The API decorator will
                    # return the original response.
                    return response

                kwargs = {**response.payload, **kwargs}

            return api_call(*args, **kwargs)

        return api_wrapper

    return decorated_patch_update


@api
def get_login():
    """Returns the login status using available credentials."""
    return requests.get('https://api.cloudns.net/dns/login.json',
                        params=get_auth_params())


@api
def get_nameservers():
    """Returns the available nameservers."""
    return requests.get(
        'https://api.cloudns.net/dns/available-name-servers.json',
        params=get_auth_params())


@api
def get_my_ip():
    """Returns the caller's ip address.

    NOTE: This doesn't seem to be working on ClouDNS's servers.
    """
    return requests.get('https://api.cloudns.net/dns/get-my-ip.json',
                        params=get_auth_params())
